@@ -62,7 +62,13 @@ else
   IS_SNAPSHOT=
 fi
 
-MINIMAL_EXCLUDE_DEPS="alsa ao bjack camlimages dssi faad fdkaac flac frei0r gd graphics gstreamer imagelib irc-client-unix ladspa lame lastfm lilv lo mad magic ogg opus osc-unix portaudio pulseaudio samplerate shine soundtouch speex srt tls theora tsdl sqlite3 vorbis"
+MINIMAL_EXCLUDE_DEPS="alsa ao bjack camlimages dssi faad fdkaac flac frei0r gd graphics irc-client-unix ladspa lame lastfm lilv lo mad magic ogg opus osc-unix portaudio pulseaudio samplerate shine soundtouch speex srt tls theora tsdl sqlite3 vorbis sdl-liquidsoap"
+
+echo "Ocaml version to build: 4.14.2, 5.2.0-ox"
+OCAML_VERSION='["4.14.2", "5.2.0-ox"]'
+
+echo "OCaml docker release version: 4.14.2"
+OCAML_DOCKER_RELEASE_VERSION="4.14.2"
 
 {
   echo "branch=${BRANCH}"
@@ -78,4 +84,6 @@ MINIMAL_EXCLUDE_DEPS="alsa ao bjack camlimages dssi faad fdkaac flac frei0r gd g
   echo "minimal_exclude_deps=${MINIMAL_EXCLUDE_DEPS}"
   echo "save_traces=${SAVE_TRACES}"
   echo "is_snapshot=${IS_SNAPSHOT}"
+  echo "ocaml_version=${OCAML_VERSION}"
+  echo "ocaml_docker_release_version=${OCAML_DOCKER_RELEASE_VERSION}"
 } >> "${GITHUB_OUTPUT}"
