@@ -40,7 +40,7 @@ echo "::endgroup::"
 echo "::group::Setting up specific dependencies"
 
 opam update
-opam install -y odoc.3.0.0~beta1
+opam upgrade -y posix-socket
 
 cd /tmp/liquidsoap-full/liquidsoap
 
@@ -52,6 +52,12 @@ export PKG_CONFIG_PATH=/usr/share/pkgconfig/pkgconfig
 
 echo "::endgroup::"
 
+echo "::group::Cleaning up cache"
+
+rm -rf /var/cache/liquidsoap/* "$HOME"/.cache/liquidsoap/*
+
+echo "::endgroup::"
+
 echo "::group::Compiling"
 
 cd /tmp/liquidsoap-full
@@ -77,7 +83,6 @@ OCAMLPATH="$(cat .ocamlpath)"
 export OCAMLPATH
 
 cd /tmp/liquidsoap-full/liquidsoap
-dune build @doc @doc-private
 dune build --profile=release
 
 echo "::endgroup::"
@@ -87,12 +92,3 @@ echo "::group::Print build config"
 dune exec -- liquidsoap --build-config
 
 echo "::endgroup::"
-
-echo "::group::Basic tests"
-
-cd /tmp/liquidsoap-full/liquidsoap
-
-dune exec -- liquidsoap --version
-dune exec -- liquidsoap --check 'print("hello world")'
-
-echo "::endgroup::"
