@@ -7,7 +7,7 @@ BASE_DIR=$(cd "${PWD}/../.." && pwd)
 
 DOCKER_IMAGE=savonet/liquidsoap-github-actions-website
 
-docker build --no-cache --tag "${DOCKER_IMAGE}" --file "${BASE_DIR}/.github/docker/Dockerfile.website" .
+docker build --no-cache --tag "${DOCKER_IMAGE}" --file "${BASE_DIR}/.github/docker/website.dockerfile" .
 
 id="$(docker create "${DOCKER_IMAGE}")"
 docker cp "$id:/tmp/liquidsoap-full/website/html" html/
