@@ -90,7 +90,9 @@ let initialized graph =
 
 let is_ready graph =
   (match (initialized graph, Queue.peek_opt graph.graph_inputs) with
-    | false, Some s -> Clock.tick s#clock
+    | false, Some s ->
+        if not (Clock.started s#clock) then Clock.start s#clock;
+        Clock.tick s#clock
     | _ -> ());
   (not graph.failed)
   && Queue.fold graph.graph_inputs
@@ -562,7 +564,7 @@ let abuffer_args frame =
   let sample_rate = Avutil.Audio.frame_get_sample_rate frame in
   let channel_layout = Avutil.Audio.frame_get_channel_layout frame in
   let channel_layout_params =
-    match Avutil.Channel_layout.get_native_id channel_layout with
+    match Avutil.Channel_layout.get_mask channel_layout with
       | Some id -> ("channel_layout", `Int64 id)
       | None ->
           let channel_layout =
@@ -570,8 +572,7 @@ let abuffer_args frame =
               (Avutil.Channel_layout.get_nb_channels channel_layout)
           in
           ( "channel_layout",
-            `Int64
-              (Option.get (Avutil.Channel_layout.get_native_id channel_layout))
+            `Int64 (Option.get (Avutil.Channel_layout.get_mask channel_layout))
           )
   in
   let sample_format = Avutil.Audio.frame_get_sample_format frame in
@@ -884,8 +885,15 @@ let _ =
       let ret = Lang.apply ~pos:(Lang.pos p) fn [("", Graph.to_value graph)] in
       let id = "ffmpeg.filter" in
       let output_clock = Clock.create ~id () in
+      let controller =
+        object
+          method id = id
+        end
+      in
       let input_clock =
-        Clock.create_sub_clock ~id:(id ^ ".input") output_clock
+        Clock.create_sub_clock ~id:(id ^ ".input")
+          ~controller:(`Other ("ffmpeg filter graph", controller))
+          output_clock
       in
       unify_clocks ~clock:input_clock graph.graph_inputs;
       unify_clocks ~clock:output_clock graph.graph_outputs;
