@@ -157,10 +157,8 @@ let _ =
       "Register a new playlist parser. An empty playlist is considered as a \
        failure to resolve."
     [
-      ( "format",
-        Lang.string_t,
-        None,
-        Some "Playlist format. If possible, a mime-type." );
+      ("name", Lang.string_t, None, Some "User-friendly format name");
+      ("mimes", Lang.(list_t string_t), None, Some "Supported mime formats.");
       ( "strict",
         Lang.bool_t,
         None,
@@ -169,7 +167,10 @@ let _ =
     ]
     Lang.unit_t
     (fun p ->
-      let format = Lang.to_string (List.assoc "format" p) in
+      let name = Lang.to_string (List.assoc "name" p) in
+      let mimes =
+        List.map Lang.to_string (Lang.to_list (List.assoc "mimes" p))
+      in
       let strict = Lang.to_bool (List.assoc "strict" p) in
       let fn = List.assoc "" p in
       let fn ?pwd uri =
@@ -187,8 +188,8 @@ let _ =
             (Lang.to_metadata_list m, Lang.to_string s))
           ret
       in
-      Plug.register Playlist_parser.parsers format ~doc:""
-        { Playlist_parser.strict; Playlist_parser.parser = fn };
+      Plug.register Playlist_parser.parsers name ~doc:""
+        { Playlist_parser.mimes; strict; parser = fn };
       Lang.unit)
 
 let default_static =
@@ -214,7 +215,7 @@ let _ =
         Some (Lang.bool false),
         Some "if true, file is removed when it is finished." );
       ( "static",
-        Lang.fun_t [(false, "", Lang.string_t)] Lang.bool_t,
+        Builtins_sys.static_t,
         Some default_static,
         Some
           "When given an uri for the protocol, if it returns `true`, then \
@@ -251,7 +252,8 @@ let _ =
       let static s = Lang.to_bool (Lang.apply static [("", Lang.string s)]) in
       let doc = Lang.to_string (List.assoc "doc" p) in
       let syntax = Lang.to_string (List.assoc "syntax" p) in
-      Lang.add_protocol ~syntax ~doc ~static name (fun arg ~log timeout ->
+      Lang.add_protocol ~syntax ~doc ~static ~mode:Request.Uri name
+        (fun arg ~log timeout ->
           let log =
             Lang.val_fun log_p (fun p ->
                 let v = List.assoc "" p in
