@@ -1,7 +1,7 @@
 (*****************************************************************************
 
-  Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2023 Savonet team
+  Liquidsoap, a programmable stream generator.
+  Copyright 2003-2024 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -24,30 +24,19 @@ exception Found of (Lang.value * Lang.value option)
 
 let settings = ref Lang.null
 
-type Type.constr_t += Dtools
-
 let dtools_constr =
   let open Liquidsoap_lang in
   let open Type in
   {
-    t = Dtools;
     constr_descr = "unit, bool, int, float, string or [string]";
     univ_descr = None;
     satisfied =
       (fun ~subtype ~satisfies:_ b ->
         let b = demeth b in
         match b.descr with
-          | Custom { typ }
-            when List.mem typ
-                   [
-                     Ground_type.Bool.Type;
-                     Ground_type.Int.Type;
-                     Ground_type.Float.Type;
-                     Ground_type.String.Type;
-                   ] ->
-              ()
+          | Bool | Int | Float | String -> ()
           | Tuple [] -> ()
-          | List { t = b } -> subtype b (make Ground_type.string)
+          | List { t = b } -> subtype b (make String)
           | _ -> raise Unsatisfied_constraint);
   }
 
@@ -188,7 +177,7 @@ type descr = {
   description : string;
   comments : string;
   children : (string * descr) list;
-  value : Lang.in_value;
+  value : Lang.value;
 }
 
 let filtered_settings = ["subordinate log level"]
@@ -197,10 +186,10 @@ let print_settings () =
   let rec grab_descr v =
     {
       description =
-        (try Lang.to_string (Value.Methods.find "description" v.Value.methods)
+        (try Lang.to_string (Value.Methods.find "description" (Value.methods v))
          with _ -> "");
       comments =
-        (try Lang.to_string (Value.Methods.find "comments" v.Value.methods)
+        (try Lang.to_string (Value.Methods.find "comments" (Value.methods v))
          with _ -> "");
       children =
         Value.Methods.fold
@@ -208,8 +197,8 @@ let print_settings () =
             if key <> "comments" && key <> "description" && key <> "set" then
               (key, grab_descr meth) :: children
             else children)
-          v.Value.methods [];
-      value = v.Value.value;
+          (Value.methods v) [];
+      value = v;
     }
   in
   let descr = grab_descr !settings in
@@ -218,25 +207,17 @@ let print_settings () =
         not (List.mem description filtered_settings))
   in
   let print_set ~path = function
-    | Value.Tuple [] -> []
-    | (Value.Fun ([], _, _) | Value.FFI ([], _)) as value ->
-        let value =
-          Lang.apply
-            {
-              Value.pos = None;
-              value;
-              methods = Value.Methods.empty;
-              id = Value.id ();
-            }
-            []
-        in
+    | Liquidsoap_lang.Value.Tuple { value = [] } -> []
+    | Liquidsoap_lang.Value.(Fun { fun_args = [] } | FFI { ffi_args = []; _ })
+      as value ->
+        let value = Lang.apply value [] in
         [
           Printf.sprintf {|
 ```liquidsoap
 %s := %s
 ```
 |} path
-            (if value.Value.value = Value.Null then "<value>"
+            (if match value with Null _ -> true | _ -> false then "<value>"
              else Value.to_string value);
         ]
     | value ->
@@ -246,13 +227,7 @@ let print_settings () =
 %s := %s
 ```
 |} path
-            (Value.to_string
-               {
-                 Value.pos = None;
-                 value;
-                 methods = Value.Methods.empty;
-                 id = Value.id ();
-               });
+            (Value.to_string value);
         ]
   in
   let rec print_descr ~level ~path descr =
@@ -278,7 +253,7 @@ let _ =
   let grab path value =
     let path = String.split_on_char '.' path in
     List.fold_left
-      (fun cur link -> Value.Methods.find link cur.Value.methods)
+      (fun cur link -> Value.Methods.find link (Value.methods cur))
       value path
   in
   ignore
@@ -323,20 +298,18 @@ let _ =
       let default = List.assoc "default" p in
       try
         let get = grab path !settings in
-        let v = Lang.apply get [] in
-        match (default.Lang.value, v.Lang.value) with
-          | Lang.(Ground (Ground.Bool _)), Lang.(Ground (Ground.Bool _))
-          | Lang.(Ground (Ground.Int _)), Lang.(Ground (Ground.Int _))
-          | Lang.(Ground (Ground.Float _)), Lang.(Ground (Ground.Float _))
-          | Lang.(Ground (Ground.String _)), Lang.(Ground (Ground.String _))
-          | Lang.(List []), Lang.(List [])
-          | ( Lang.(List ({ pos = _; value = Ground (Ground.String _) } :: _)),
-              Lang.(List []) )
-          | ( Lang.(List []),
-              Lang.(List ({ pos = _; value = Ground (Ground.String _) } :: _)) )
-          | ( Lang.(List ({ pos = _; value = Ground (Ground.String _) } :: _)),
-              Lang.(List ({ pos = _; value = Ground (Ground.String _) } :: _)) )
-            ->
+        let v = Lang.apply ~pos:(Lang.pos p) get [] in
+        let open Liquidsoap_lang.Value in
+        match (default, v) with
+          | Bool _, Bool _
+          | Int _, Int _
+          | Float _, Float _
+          | String _, String _
+          | List { value = [] }, List { value = [] }
+          | List { value = Value.String _ :: _ }, List { value = [] }
+          | List { value = [] }, List { value = Value.String _ :: _ }
+          | ( List { value = Value.String _ :: _ },
+              List { value = Value.String _ :: _ } ) ->
               v
           | _ ->
               log#severe
