@@ -20,6 +20,8 @@
 
  *****************************************************************************)
 
+let log = Log.make ["tls"]
+
 module Http = Liq_http
 
 module Liq_tls = struct
@@ -158,27 +160,51 @@ module Liq_tls = struct
     Unix.close h.fd
 end
 
-let tls_socket ~session transport =
-  object
-    method typ = "tls"
-    method transport = transport
-    method file_descr = session.Liq_tls.fd
-
-    method wait_for ?log event timeout =
-      let event =
-        match event with
-          | `Read -> `Read session.Liq_tls.fd
-          | `Write -> `Write session.Liq_tls.fd
-          | `Both -> `Both session.Liq_tls.fd
+let tls_socket ~pos ~session transport =
+  let closed = Atomic.make false in
+  let finalise s =
+    if not (Atomic.get closed) then (
+      let pos =
+        match pos with
+          | [] -> "unknown"
+          | _ ->
+              String.concat ", " (List.map (fun pos -> Pos.to_string pos) pos)
       in
-      Tutils.wait_for ?log event timeout
+      log#critical
+        "SSL socket closed during garbage collection, you must have a leak in \
+         your application! Socket opened at position: %s"
+        pos;
+      try s#close with _ -> ())
+  in
+  let s =
+    object
+      method typ = "tls"
+      method transport = transport
+      method file_descr = session.Liq_tls.fd
 
-    method read = Liq_tls.read session
-    method write = Liq_tls.write session
-    method close = Liq_tls.close session
-  end
+      method wait_for ?log event timeout =
+        let event =
+          match event with
+            | `Read -> `Read session.Liq_tls.fd
+            | `Write -> `Write session.Liq_tls.fd
+            | `Both -> `Both session.Liq_tls.fd
+        in
+        Tutils.wait_for ?log event timeout
 
-let server ~read_timeout ~write_timeout ~certificate ~key transport =
+      method read = Liq_tls.read session
+      method write = Liq_tls.write session
+      method closed = Atomic.get closed
+
+      method close =
+        Atomic.set closed true;
+        Liq_tls.close session
+    end
+  in
+  Gc.finalise finalise s;
+  s
+
+let server ~read_timeout ~write_timeout ~certificate ~key ~client_certificate
+    transport =
   let server =
     try
       let certificate = Utils.read_all (certificate ()) in
@@ -188,8 +214,21 @@ let server ~read_timeout ~write_timeout ~certificate ~key transport =
       let key =
         Result.get_ok (X509.Private_key.decode_pem (Utils.read_all (key ())))
       in
+      let authenticator =
+        match client_certificate () with
+          | None -> None
+          | Some cert ->
+              Some
+                (X509.Authenticator.chain_of_trust
+                   ~time:(fun () -> Some (Ptime_clock.now ()))
+                   (Result.get_ok
+                      (X509.Certificate.decode_pem_multiple
+                         (Utils.read_all cert))))
+      in
       match
-        Tls.Config.server ~certificates:(`Single (certificates, key)) ()
+        Tls.Config.server
+          ~certificates:(`Single (certificates, key))
+          ?authenticator ()
       with
         | Ok server -> server
         | Error (`Msg message) -> Runtime_error.raise ~pos:[] ~message "tls"
@@ -206,14 +245,15 @@ let server ~read_timeout ~write_timeout ~certificate ~key transport =
         Http.set_socket_default ~read_timeout:timeout ~write_timeout:timeout fd;
         let session = Liq_tls.init_server ~timeout ~server fd in
         Http.set_socket_default ~read_timeout ~write_timeout fd;
-        (tls_socket ~session transport, caller)
+        (tls_socket ~pos:[] ~session transport, caller)
       with exn ->
         let bt = Printexc.get_raw_backtrace () in
         Unix.close fd;
         Printexc.raise_with_backtrace exn bt
   end
 
-let transport ~read_timeout ~write_timeout ~certificate ~key () =
+let transport ~read_timeout ~write_timeout ~certificate ~key ~client_certificate
+    ~client_key () =
   object (self)
     method name = "tls"
     method protocol = "https"
@@ -242,16 +282,31 @@ let transport ~read_timeout ~write_timeout ~certificate ~key () =
               let r = auth ?ip ~host certs in
               if Result.is_ok r then r else authenticator ?ip ~host certs
       in
+      let certificates =
+        match client_certificate () with
+          | None -> `None
+          | Some cert ->
+              `Single
+                ( Result.get_ok
+                    (X509.Certificate.decode_pem_multiple (Utils.read_all cert)),
+                  Result.get_ok
+                    (X509.Private_key.decode_pem
+                       (Utils.read_all (client_key ()))) )
+      in
       let client =
-        match Tls.Config.client ~authenticator ~peer_name:domain () with
+        match
+          Tls.Config.client ~authenticator ~certificates ~peer_name:domain ()
+        with
           | Ok client -> client
           | Error (`Msg message) -> Runtime_error.raise ~pos:[] ~message "tls"
       in
       let fd = Http.connect ?bind_address ~timeout ?prefer host port in
       let session = Liq_tls.init_client ~timeout ~client fd in
-      tls_socket ~session self
+      tls_socket ~pos:[] ~session self
 
-    method server = server ~read_timeout ~write_timeout ~certificate ~key self
+    method server =
+      server ~read_timeout ~write_timeout ~certificate ~key ~client_certificate
+        self
   end
 
 let _ =
@@ -279,6 +334,20 @@ let _ =
         Some
           "Path to certificate private key. Required in server mode, e.g. \
            `input.harbor`, etc. Unused in client mode." );
+      ( "client_certificate",
+        Lang.nullable_t Lang.string_t,
+        Some Lang.null,
+        Some
+          "Path to client certificate file. If passed in server mode, clients \
+           will be required to present a certificate from this file. If passed \
+           in client mode, the first certificate in this file will be \
+           presented to the server." );
+      ( "client_key",
+        Lang.nullable_t Lang.string_t,
+        Some Lang.null,
+        Some
+          "Path to client certificate private key. Required in client mode if \
+           a client certificate is passed. Unused in server mode." );
     ]
     Lang.http_transport_t
     (fun p ->
@@ -303,5 +372,13 @@ let _ =
       in
       let certificate = find "certificate" in
       let key = find "key" in
+      let find_opt name () =
+        match Lang.to_valued_option Lang.to_string (List.assoc name p) with
+          | None -> None
+          | Some path -> Some (Utils.check_readable ~pos:(Lang.pos p) path)
+      in
+      let client_certificate = find_opt "client_certificate" in
+      let client_key = find "client_key" in
       Lang.http_transport
-        (transport ~read_timeout ~write_timeout ~certificate ~key ()))
+        (transport ~read_timeout ~write_timeout ~certificate ~key
+           ~client_certificate ~client_key ()))
