@@ -71,18 +71,23 @@ let conf_clock_preferred =
     ~p:(conf_clock#plug "preferred")
     "Preferred clock implementation. One if: \"posix\" or \"ocaml\"."
 
-let conf_clock_sleep_latency =
-  Dtools.Conf.int
-    ~p:(conf_clock#plug "sleep_latency")
-    ~d:5
-    "How much time ahead (in frame duration) we should be until we let the \
-     streaming loop sleep."
+let conf_clock_latency =
+  Dtools.Conf.float
+    ~p:(conf_clock#plug "latency")
+    ~d:0.1
+    "How much time ahead (in seconds) we should be until we let the streaming \
+     loop rest."
     ~comments:
       [
         "Once we have computed the given amount of time time in advance,";
         "we wait until re-starting the streaming loop.";
       ]
 
+let conf_leak_warning =
+  Dtools.Conf.int
+    ~p:(conf_clock#plug "leak_warning")
+    ~d:50 "Number of sources at which a leak warning should be issued."
+
 let time_implementation () =
   try Hashtbl.find Liq_time.implementations conf_clock_preferred#get
   with Not_found -> Liq_time.unix
@@ -122,7 +127,7 @@ type state =
 
 type clock = {
   id : string option Unifier.t;
-  sub_ids : string list;
+  controller : controller Unifier.t;
   stack : Pos.t list Atomic.t;
   state : state Atomic.t;
   pending_activations : source Queue.t;
@@ -131,21 +136,63 @@ type clock = {
 }
 
 and t = clock Unifier.t
+and controller = [ `None | `Clock of t | `Other of string * < id : string > ]
 
-let _default_id { id; pending_activations } =
-  match (Unifier.deref id, Queue.elements pending_activations) with
+let string_of_state = function
+  | `Stopping _ -> "stopping"
+  | `Started _ -> "started"
+  | `Stopped _ -> "stopped"
+
+let meaningful_pending_id pending =
+  match
+    List.sort
+      (fun s s' ->
+        match (s#source_type, s'#source_type) with
+          | `Output _, `Output _ -> 0
+          | `Output _, _ -> -1
+          | _, `Output _ -> 1
+          | `Active _, `Active _ -> 0
+          | `Active _, _ -> -1
+          | _, `Active _ -> 1
+          | `Passive, `Passive -> 0)
+      pending
+  with
+    | el :: _ -> Some el#id
+    | _ -> None
+
+let _id { id; pending_activations } =
+  match
+    ( Unifier.deref id,
+      meaningful_pending_id (Queue.elements pending_activations) )
+  with
     | Some id, _ -> id
-    | None, el :: _ -> el#id
-    | None, _ -> "generic"
-
-let _id clock =
-  _default_id clock
-  ^ match clock.sub_ids with [] -> "" | l -> "." ^ String.concat "." l
+    | None, Some id -> id
+    | _ -> "generic"
 
 let id c = _id (Unifier.deref c)
+let generate_id = Lang_string.generate_id ~category:"clock"
 
-let set_id c id =
-  Unifier.set (Unifier.deref c).id (Some (Lang_string.generate_id id))
+let string_of_controller = function
+  | `None -> "none"
+  | `Clock c -> Printf.sprintf "clock %s" (id c)
+  | `Other (c, o) -> Printf.sprintf "%s %s" c o#id
+
+let unifiable_controller ~unify c c' =
+  match (c, c') with
+    | `None, _ | _, `None -> true
+    | `Other (c, o), `Other (c', o') -> c = c' && o == o'
+    | `Clock c, `Clock c' -> (
+        try
+          unify c c';
+          true
+        with _ -> false)
+    | _ -> false
+
+let _set_id _clock new_id =
+  if Unifier.deref _clock.id <> Some new_id then
+    Unifier.set _clock.id (Some (generate_id new_id))
+
+let set_id clock new_id = _set_id (Unifier.deref clock) new_id
 
 let attach c s =
   let clock = Unifier.deref c in
@@ -205,14 +252,11 @@ let _sync ?(pending = false) x =
     | `Started { sync } -> (sync :> sync_mode)
 
 let sync c = _sync (Unifier.deref c)
-let cleanup_source s = try s#force_sleep with _ -> ()
 let pending_clocks = WeakQueue.create ()
 let clocks = Queue.create ()
 
-let rec _cleanup ~clock { outputs; passive_sources; active_sources } =
-  Queue.iter outputs cleanup_source;
-  WeakQueue.iter passive_sources cleanup_source;
-  WeakQueue.iter active_sources cleanup_source;
+let rec _cleanup ~clock { outputs } =
+  Queue.iter outputs (fun o -> try o#sleep o with _ -> ());
   Queue.iter clock.sub_clocks stop;
   Queue.filter_out clocks (fun c -> Unifier.deref c == clock)
 
@@ -222,10 +266,10 @@ and stop c =
     | `Stopped _ | `Stopping _ -> ()
     | `Started ({ sync = `Passive } as x) ->
         _cleanup ~clock x;
-        x.log#debug "Clock stopped";
+        x.log#important "Clock stopped";
         Atomic.set clock.state (`Stopped `Passive)
     | `Started x ->
-        x.log#debug "Clock stopping";
+        x.log#important "Clock stopping";
         Atomic.set clock.state (`Stopping x)
 
 let clocks_started = Atomic.make false
@@ -246,9 +290,14 @@ let descr clock =
       | _ -> "")
 
 let unify =
-  let unify c c' =
+  let _unify c c' =
     let clock = Unifier.deref c in
     let clock' = Unifier.deref c' in
+    (match
+       (Unifier.deref clock.controller, Unifier.deref clock'.controller)
+     with
+      | _, `None -> Unifier.(clock'.controller <-- clock.controller)
+      | _ -> ());
     Queue.flush_iter clock.pending_activations
       (Queue.push clock'.pending_activations);
     Queue.flush_iter clock.sub_clocks (Queue.push clock'.sub_clocks);
@@ -258,27 +307,43 @@ let unify =
       | Some _, None -> Unifier.(clock'.id <-- clock.id)
       | None, Some _ -> Unifier.(clock.id <-- clock'.id)
       | Some _, Some id ->
-          log#important
-            "Clocks %s and %s both have id already set. Setting id to %s"
+          log#info "Clocks %s and %s both have id already set. Setting id to %s"
             (descr c) (descr c') id;
           Unifier.(clock.id <-- clock'.id));
     Unifier.(c <-- c');
-    Queue.filter_out clocks (fun el -> el == c)
+    Queue.filter_out clocks (fun el -> el == c);
+    WeakQueue.filter_out pending_clocks (fun el -> el == c)
   in
-  fun ~pos c c' ->
+  let rec unify ~pos c c' =
     let _c = Unifier.deref c in
     let _c' = Unifier.deref c' in
+    let _controller = Unifier.deref _c.controller in
+    let _controller' = Unifier.deref _c'.controller in
+    if not (unifiable_controller ~unify:(unify ~pos) _controller _controller')
+    then
+      raise
+        Liquidsoap_lang.Error.(
+          Clock_main
+            {
+              pos;
+              left_main = string_of_controller _controller;
+              left_child = descr c;
+              right_main = string_of_controller _controller';
+              right_child = descr c';
+            });
     match (_c == _c', Atomic.get _c.state, Atomic.get _c'.state) with
       | true, _, _ -> ()
-      | _, `Stopped s, `Stopped s' when s = s' -> unify c c'
+      | _, `Stopped s, `Stopped s' when s = s' -> _unify c c'
       | _, `Stopped s, _
         when s = `Automatic || (s :> sync_mode) = _sync ~pending:true _c' ->
-          unify c c'
+          _unify c c'
       | _, _, `Stopped s'
         when s' = `Automatic || _sync ~pending:true _c = (s' :> sync_mode) ->
-          unify c' c
+          _unify c' c
       | _ ->
           raise (Liquidsoap_lang.Error.Clock_conflict (pos, descr c, descr c'))
+  in
+  unify
 
 let () =
   Lifecycle.before_core_shutdown ~name:"Clocks stop" (fun () ->
@@ -336,7 +401,7 @@ let _set_time { time_implementation; t0; frame_duration; ticks } t =
   let delta = Time.(to_float (t |-| t0)) in
   Atomic.set ticks (int_of_float (delta /. Time.to_float frame_duration))
 
-let _after_tick ~clock x =
+let _after_tick ~self_sync x =
   Queue.flush_iter x.after_tick (fun fn ->
       check_stopped ();
       fn ());
@@ -344,13 +409,11 @@ let _after_tick ~clock x =
   let end_time = Time.time () in
   let target_time = _target_time x in
   check_stopped ();
-  match (x.sync, _self_sync ~clock x, Time.(end_time |<| target_time)) with
+  match (x.sync, self_sync, Time.(end_time |<| target_time)) with
     | `Unsynced, _, _ | `Passive, _, _ | `Automatic, true, _ -> ()
     | `Automatic, false, true | `CPU, _, true ->
         if
-          Time.(
-            of_float (float conf_clock_sleep_latency#get)
-            |*| x.frame_duration |<=| (target_time |-| end_time))
+          Time.(of_float conf_clock_latency#get |<=| (target_time |-| end_time))
         then Time.sleep_until target_time
     | _ ->
         let latency = Time.(end_time |-| target_time) in
@@ -370,7 +433,12 @@ let _after_tick ~clock x =
             && x.log_delay |<=| (end_time |-| Atomic.get x.last_catchup_log))
         then (
           Atomic.set x.last_catchup_log end_time;
-          x.log#severe "We must catchup %.2f seconds!"
+          x.log#severe
+            "Latency is too high: we must catchup %.2f seconds! Check if your \
+             system can process your stream fast enough (CPU usage, disk \
+             access, etc) or if your stream should be self-sync (can happen \
+             when using `input.ffmpeg`). Refer to the latency control section \
+             of the documentation for more info."
             Time.(to_float (end_time |-| target_time)))
 
 let started c =
@@ -396,23 +464,56 @@ let rec active_params c =
   match Atomic.get (Unifier.deref c).state with
     | `Stopping s | `Started s -> s
     | _ when Atomic.get global_stop -> raise Has_stopped
-    | _ -> raise Invalid_state
+    | s ->
+        log#critical "Clock %s has invalid state: %s" (id c) (string_of_state s);
+        raise Invalid_state
 
 and _activate_pending_sources ~clock x =
+  let pending_sources = Queue.length clock.pending_activations in
   Queue.flush_iter clock.pending_activations
     (wrap_errors clock (fun s ->
          check_stopped ();
-         s#wake_up;
          match s#source_type with
            | `Active _ -> WeakQueue.push x.active_sources s
-           | `Output _ -> Queue.push x.outputs s
-           | `Passive -> WeakQueue.push x.passive_sources s))
+           | `Output _ ->
+               s#wake_up s;
+               Queue.push x.outputs s
+           | `Passive -> WeakQueue.push x.passive_sources s));
+  if 0 < pending_sources then (
+    let total_sources =
+      Queue.length x.outputs
+      + WeakQueue.length x.active_sources
+      + WeakQueue.length x.passive_sources
+    in
+    if
+      (total_sources - pending_sources) / conf_leak_warning#get
+      < total_sources / conf_leak_warning#get
+    then (
+      x.log#severe
+        "There are currently %d sources, possible source leak! Please check \
+         that you don't have a loop creating multiple sources."
+        total_sources;
+      let ids =
+        List.map
+          (fun s ->
+            let source_type =
+              match s#source_type with
+                | `Passive -> "passive"
+                | `Active _ -> "active"
+                | `Output _ -> "output"
+            in
+            Printf.sprintf "%s (%s)" s#id source_type)
+          (Queue.elements x.outputs
+          @ WeakQueue.elements x.active_sources
+          @ WeakQueue.elements x.passive_sources)
+      in
+      x.log#important "Current sources: %s" (String.concat ", " ids)))
 
 and _tick ~clock x =
-  _activate_pending_sources ~clock x;
   let sub_clocks =
     List.map (fun c -> (c, ticks c)) (Queue.elements clock.sub_clocks)
   in
+  _activate_pending_sources ~clock x;
   let sources = _animated_sources x in
   List.iter
     (wrap_errors clock (fun s ->
@@ -424,6 +525,8 @@ and _tick ~clock x =
   Queue.flush_iter x.on_tick (fun fn ->
       check_stopped ();
       fn ());
+  let self_sync = _self_sync ~clock x in
+  check_stopped ();
   List.iter
     (fun (c, old_ticks) ->
       if ticks c = old_ticks then
@@ -431,7 +534,7 @@ and _tick ~clock x =
     sub_clocks;
   Atomic.incr x.ticks;
   check_stopped ();
-  _after_tick ~clock x;
+  _after_tick ~self_sync x;
   check_stopped ()
 
 and _clock_thread ~clock x =
@@ -440,30 +543,44 @@ and _clock_thread ~clock x =
     || 0 < Queue.length x.outputs
     || 0 < WeakQueue.length x.active_sources
   in
+  let clock_stopped () =
+    match Atomic.get clock.state with `Started _ -> true | _ -> false
+  in
+  let global_stop () = Atomic.get global_stop in
   let on_stop () =
-    x.log#info "Clock thread has stopped";
+    let reasons =
+      [
+        ("clock stopped", clock_stopped ());
+        ("global stop", global_stop ());
+        ("no more sources to process", has_sources_to_process ());
+      ]
+    in
+    let reasons =
+      List.fold_left
+        (fun reasons -> function
+          | _, false -> reasons | reason, true -> reason :: reasons)
+        [] reasons
+    in
+    x.log#important "Clock thread has stopped: %s." (String.concat ", " reasons);
     _cleanup ~clock x;
     Atomic.set clock.state (`Stopped x.sync)
   in
   let run () =
     try
       while
-        (match Atomic.get clock.state with `Started _ -> true | _ -> false)
-        && (not (Atomic.get global_stop))
-        && has_sources_to_process ()
+        clock_stopped () && (not (global_stop ())) && has_sources_to_process ()
       do
         _tick ~clock x
       done;
       on_stop ()
     with Has_stopped -> on_stop ()
   in
-  ignore
-    (Tutils.create
-       (fun () ->
-         x.log#info "Clock thread is starting";
-         run ())
-       ()
-       ("Clock " ^ _id clock))
+  Tutils.create
+    (fun () ->
+      x.log#info "Clock thread is starting";
+      run ())
+    ()
+    ("Clock " ^ _id clock)
 
 and _can_start ?(force = false) clock =
   let has_output =
@@ -480,11 +597,36 @@ and _can_start ?(force = false) clock =
     | _ -> `False
 
 and _start ?force ~sync clock =
-  Unifier.set clock.id (Some (Lang_string.generate_id (_default_id clock)));
+  _set_id clock (_id clock);
   let id = _id clock in
-  log#important "Starting clock %s with %d source(s) and sync: %s" id
-    (Queue.length clock.pending_activations)
-    (string_of_sync_mode sync);
+  let sources =
+    List.fold_left
+      (fun sources s ->
+        let source_type =
+          match s#source_type with
+            | `Passive -> "passive"
+            | `Active _ -> "active"
+            | `Output _ -> "output"
+        in
+        Printf.sprintf "%s (%s)" s#id source_type :: sources)
+      []
+      (Queue.elements clock.pending_activations)
+  in
+  let sources = String.concat ", " sources in
+  let top_level, sync_mode =
+    match sync with
+      | `Passive -> ("passive", "")
+      | _ ->
+          ( "top-level",
+            Printf.sprintf " and sync: %s" (string_of_sync_mode sync) )
+  in
+  let _controller = Unifier.deref clock.controller in
+  let controlled_by =
+    if _controller = `None then ""
+    else Printf.sprintf " controlled by %s" (string_of_controller _controller)
+  in
+  log#important "Starting %s clock %s%s with sources: %s%s" top_level id
+    controlled_by sources sync_mode;
   let time_implementation = time_implementation () in
   let module Time = (val time_implementation : Liq_time.T) in
   let frame_duration = Time.of_float (Lazy.force Frame.duration) in
@@ -515,7 +657,17 @@ and _start ?force ~sync clock =
   in
   Queue.iter clock.sub_clocks (fun c -> start ?force c);
   Atomic.set clock.state (`Started x);
-  if sync <> `Passive then _clock_thread ~clock x
+  if sync <> `Passive then (
+    let th = _clock_thread ~clock x in
+    match _controller with
+      | `None ->
+          let controller =
+            object
+              method id = string_of_int (Thread.id th)
+            end
+          in
+          Unifier.set clock.controller (`Other ("thread", controller))
+      | _ -> raise Invalid_state)
 
 and start ?force c =
   let clock = Unifier.deref c in
@@ -539,14 +691,15 @@ let add_pending_clock =
     Gc.finalise finalise c;
     WeakQueue.push pending_clocks c
 
-let create ?(stack = []) ?on_error ?id ?(sub_ids = []) ?(sync = `Automatic) () =
+let create ?(stack = []) ?(controller = `None) ?on_error ?id
+    ?(sync = `Automatic) () =
   let on_error_queue = Queue.create () in
   (match on_error with None -> () | Some fn -> Queue.push on_error_queue fn);
   let c =
     Unifier.make
       {
-        id = Unifier.make (Option.map Lang_string.generate_id id);
-        sub_ids;
+        id = Unifier.make (Option.map generate_id id);
+        controller = Unifier.make controller;
         stack = Atomic.make stack;
         pending_activations = Queue.create ();
         sub_clocks = Queue.create ();
@@ -558,9 +711,11 @@ let create ?(stack = []) ?on_error ?id ?(sub_ids = []) ?(sync = `Automatic) () =
   c
 
 let time c =
-  let ({ time_implementation } as c) = active_params c in
-  let module Time = (val time_implementation : Liq_time.T) in
-  Time.to_float (_time c)
+  match active_params c with
+    | { time_implementation } as c ->
+        let module Time = (val time_implementation : Liq_time.T) in
+        Time.to_float (_time c)
+    | exception Invalid_state -> -1.
 
 let start_pending () =
   let c = WeakQueue.flush_elements pending_clocks in
@@ -608,17 +763,17 @@ let tick clock = _tick ~clock:(Unifier.deref clock) (active_params clock)
 let set_stack c stack =
   ignore (Atomic.compare_and_set (Unifier.deref c).stack [] stack)
 
-let create_sub_clock ~id clock =
-  let clock = Unifier.deref clock in
+let create_sub_clock ?controller ~id clock =
+  let controller = Option.value ~default:(`Clock clock) controller in
+  let _clock = Unifier.deref clock in
   let sub_clock =
-    create ~stack:(Atomic.get clock.stack) ~id
-      ~sub_ids:(clock.sub_ids @ ["child"])
-      ~sync:`Passive ()
+    create ~stack:(Atomic.get _clock.stack) ~id ~controller ~sync:`Passive ()
   in
-  Queue.push clock.sub_clocks sub_clock;
+  Queue.push _clock.sub_clocks sub_clock;
   sub_clock
 
-let create ?stack ?on_error ?id ?sync () = create ?stack ?on_error ?id ?sync ()
+let create ?stack ?controller ?on_error ?id ?sync () =
+  create ?stack ?controller ?on_error ?id ?sync ()
 
 let clocks () =
   List.sort_uniq
