@@ -129,10 +129,12 @@ type source =
   ; self_sync : self_sync
   ; source_type : source_type
   ; active : bool
-  ; wake_up : unit
-  ; force_sleep : unit
+  ; wake_up : 'a -> unit
+  ; sleep : 'a -> unit
   ; is_ready : bool
   ; get_frame : Frame.t >
+  as
+  'a
 
 let self_sync_type sources =
   Lazy.from_fun (fun () ->
