@@ -517,7 +517,9 @@ let parse_encoder_params =
   fun s ->
     let lexbuf = Sedlexing.Utf8.from_string ("(" ^ s ^ ")") in
     let tokenizer = Liquidsoap_lang.Preprocessor.mk_tokenizer lexbuf in
-    Liquidsoap_lang.Term_reducer.to_encoder_params (processor tokenizer)
+    Liquidsoap_lang.Term_reducer.to_encoder_params
+      ~throw:(fun ~bt exn -> Printexc.raise_with_backtrace exn bt)
+      (processor tokenizer)
 
 let parse_input_args args =
   try
@@ -727,11 +729,12 @@ let get_type ~ctype ~format ~url container =
                    (Ffmpeg_copy_content.lift_params (Some (`Audio p))));
               Frame.Fields.add field format content_type
           | p, Some format when Ffmpeg_raw_content.Audio.is_format format ->
-              ignore
-                (Content.merge format
-                   Ffmpeg_raw_content.(
-                     Audio.lift_params (AudioSpecs.mk_params p)));
-              Frame.Fields.add field format content_type
+              let dst_format =
+                Ffmpeg_raw_content.(Audio.lift_params (AudioSpecs.mk_params p))
+              in
+              (try ignore (Content.merge format dst_format)
+               with _ when Content.compatible format dst_format -> ());
+              Frame.Fields.add field dst_format content_type
           | p, Some format ->
               Frame.Fields.add field
                 (Frame_base.format_of_channels ~pcm_kind:(Content.kind format)
@@ -919,7 +922,7 @@ let mk_decoder ~streams ~target_position container =
           | _ -> ()
       with
         | Avutil.Error `Eagain | Avutil.Error `Invalid_data -> f ()
-        | Avutil.Error `Eof ->
+        | Avutil.Error `Exit | Avutil.Error `Eof ->
             Generator.add_track_mark buffer.Decoder.generator;
             raise End_of_file
         | exn ->
