@@ -109,8 +109,8 @@ let priority =
     "Priority for the flac decoder" ~d:1
 
 (* Get the number of channels of audio in a flac file.
- * This is done by decoding a first chunk of data, thus checking
- * that libmad can actually open the file -- which doesn't mean much. *)
+   This is done by decoding a first chunk of data, thus checking
+   that libmad can actually open the file -- which doesn't mean much. *)
 let file_type filename =
   let fd = Decoder.openfile filename in
   Fun.protect
