@@ -109,7 +109,7 @@ let encoder avi =
     else ans
   in
   {
-    Encoder.insert_metadata = (fun _ -> ());
+    Encoder.encode_metadata = (fun _ -> ());
     hls = Encoder.dummy_hls encode;
     encode;
     header = (fun () -> Strings.of_string header);
