@@ -281,8 +281,13 @@ let encoder ~pos ~on_keyframe ~keyframes ~mk_streams ffmpeg meta =
   in
   let split_encode frame =
     let encoder = Atomic.get encoder in
+    let encoded_fields = List.map fst (Frame.Fields.bindings encoder.streams) in
     let can_split () =
-      List.for_all (fun (_, keyframe) -> Atomic.get keyframe) keyframes
+      List.for_all
+        (fun (field, keyframe) ->
+          if not (List.mem field encoded_fields) then true
+          else Atomic.get keyframe)
+        keyframes
     in
     let flushed =
       if can_split () then Atomic.make (Some (Strings.Mutable.flush buf))
@@ -312,7 +317,7 @@ let encoder ~pos ~on_keyframe ~keyframes ~mk_streams ffmpeg meta =
   let flush () =
     Frame.Fields.iter (fun _ { flush } -> flush ()) (Atomic.get encoder).streams
   in
-  let insert_metadata m =
+  let encode_metadata m =
     let m = Frame.Metadata.to_list (Frame.Metadata.Export.to_metadata m) in
     match (ffmpeg.Ffmpeg_format.output, ffmpeg.Ffmpeg_format.format) with
       | _ when not (Atomic.get (Atomic.get encoder).started) ->
@@ -324,7 +329,7 @@ let encoder ~pos ~on_keyframe ~keyframes ~mk_streams ffmpeg meta =
           Av.set_output_metadata (Atomic.get encoder).output m
       | _ -> ()
   in
-  insert_metadata meta;
+  encode_metadata meta;
   let stop () =
     flush ();
     (try Av.close (Atomic.get encoder).output with _ -> ());
@@ -342,7 +347,7 @@ let encoder ~pos ~on_keyframe ~keyframes ~mk_streams ffmpeg meta =
     }
   in
   {
-    Encoder.insert_metadata;
+    Encoder.encode_metadata;
     header = (fun () -> Strings.empty);
     hls;
     encode;
