@@ -1,7 +1,7 @@
 (*****************************************************************************
 
-  Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2023 Savonet team
+  Liquidsoap, a programmable stream generator.
+  Copyright 2003-2024 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -22,13 +22,12 @@
 
 open Liquidsoap_lang
 open Term
-open Term.Ground
 
-module V = Lang_core.MkAbstract (struct
+module V = Lang_core.MkCustom (struct
   type content = Encoder.format
 
   let name = "encoder"
-  let descr = Encoder.string_of_format
+  let to_string = Encoder.string_of_format
 
   let to_json ~pos _ =
     Runtime_error.raise ~pos
@@ -48,28 +47,19 @@ module L = struct
   let format = V.to_value
 end
 
-let has_started = ref false
-
-let () =
-  Lifecycle.before_start ~name:"lang_encoder initialization" (fun () ->
-      has_started := true)
-
 let raise_error ~pos message =
-  match !has_started with
-    | false -> raise (Lang_error.Encoder_error (pos, message))
-    | true ->
-        Runtime_error.raise ~message
-          ~pos:(match pos with None -> [] | Some pos -> [pos])
-          "encoder"
+  Runtime_error.raise ~message
+    ~pos:(match pos with None -> [] | Some pos -> [pos])
+    "encoder"
 
-let raise_generic_error (l, t) =
-  match t with
-    | `Value v ->
-        raise_error ~pos:v.Value.pos
-          (Printf.sprintf
-             "unknown parameter name (%s) or invalid parameter value (%s)" l
-             (Value.to_string v))
-    | `Encoder _ -> raise_error ~pos:None "unexpected subencoder"
+let raise_generic_error = function
+  | `Anonymous s -> raise_error ~pos:None ("Unknown encoder parameter: " ^ s)
+  | `Labelled (l, v) ->
+      raise_error ~pos:(Value.pos v)
+        (Printf.sprintf
+           "unknown parameter name (%s) or invalid parameter value (%s)" l
+           (Value.to_string v))
+  | `Encoder _ -> raise_error ~pos:None "unexpected subencoder"
 
 (* An encoder. *)
 type encoder = {
@@ -92,26 +82,25 @@ let channels_of_params ?(default = 2) p =
   match
     List.find_map
       (function
-        | "", `Term { term = Ground (String "stereo") } -> Some 2
-        | "", `Term { term = Ground (String "mono") } -> Some 1
-        | "stereo", `Term { term = Ground (Bool b); _ } ->
+        | `Anonymous s when String.lowercase_ascii s = "mono" -> Some 1
+        | `Anonymous s when String.lowercase_ascii s = "stereo" -> Some 2
+        | `Labelled ("stereo", { term = `Bool b; _ }) ->
             Some (if b then 2 else 1)
-        | "stereo", `Term ({ t = { Type.pos } } as tm) ->
+        | `Labelled ("stereo", ({ t = { Type.pos } } as tm)) ->
             raise_error ~pos
               (Printf.sprintf
                  "Invalid value %s for stereo mode. Only static `true` or \
                   `false` are allowed."
                  (Term.to_string tm))
-        | "mono", `Term { term = Ground (Bool b); _ } ->
-            Some (if b then 1 else 2)
-        | "mono", `Term ({ t = { Type.pos } } as tm) ->
+        | `Labelled ("mono", { term = `Bool b; _ }) -> Some (if b then 1 else 2)
+        | `Labelled ("mono", ({ t = { Type.pos } } as tm)) ->
             raise_error ~pos
               (Printf.sprintf
                  "Invalid value %s for mono mode. Only static `true` or \
                   `false` are allowed."
                  (Term.to_string tm))
-        | "channels", `Term { term = Ground (Int n) } -> Some n
-        | "channels", `Term ({ t = { Type.pos } } as tm) ->
+        | `Labelled ("channels", { term = `Int n }) -> Some n
+        | `Labelled ("channels", ({ t = { Type.pos } } as tm)) ->
             raise_error ~pos
               (Printf.sprintf
                  "Invalid value %s for channels mode. Only static numbers are \
@@ -132,11 +121,9 @@ let type_of_encoder ~pos e =
   let frame_t = Frame_type.make Liquidsoap_lang.Lang.unit_t fields in
   L.format_t ?pos frame_t
 
-let make_encoder ~pos t ((e, p) : Hooks.encoder) =
+let make_encoder ~pos ((e, p) : Hooks.encoder) =
   try
     let e = (find_encoder e).make p in
     let (_ : Encoder.factory) = Encoder.get_factory e in
     V.to_value ?pos e
-  with Not_found ->
-    raise_error ~pos
-      (Printf.sprintf "unsupported format: %s" (Term.to_string t))
+  with Not_found -> raise_error ~pos "unsupported format"
