@@ -1,7 +1,7 @@
 (*****************************************************************************
 
-  Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2023 Savonet team
+  Liquidsoap, a programmable stream generator.
+  Copyright 2003-2024 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -27,11 +27,11 @@ type mode = Low_pass | High_pass | Band_pass | Notch
 class filter (source : source) freq q wet mode =
   let rate = float (Lazy.force Frame.audio_rate) in
   object (self)
-    inherit operator ~name:"filter" [source] as super
-    method stype = source#stype
+    inherit operator ~name:"filter" [source]
+    method fallible = source#fallible
     method remaining = source#remaining
     method seek_source = source#seek_source
-    method private _is_ready = source#is_ready
+    method private can_generate_frame = source#is_ready
     method abort_track = source#abort_track
     method self_sync = source#self_sync
     val mutable low = [||]
@@ -39,13 +39,13 @@ class filter (source : source) freq q wet mode =
     val mutable band = [||]
     val mutable notch = [||]
 
-    method! wake_up a =
-      super#wake_up a;
-      let channels = self#audio_channels in
-      low <- Array.make channels 0.;
-      high <- Array.make channels 0.;
-      band <- Array.make channels 0.;
-      notch <- Array.make channels 0.
+    initializer
+      self#on_wake_up (fun () ->
+          let channels = self#audio_channels in
+          low <- Array.make channels 0.;
+          high <- Array.make channels 0.;
+          band <- Array.make channels 0.;
+          notch <- Array.make channels 0.)
 
     (* State vartiable filter, see
        http://www.musicdsp.org/archive.php?classid=3#23
@@ -56,18 +56,17 @@ class filter (source : source) freq q wet mode =
 
        Maybe should we implement Chamberlin's version instead, which handles freq
        <= rate/2. See http://www.musicdsp.org/archive.php?classid=3#142 *)
-    method private get_frame buf =
-      let offset = AFrame.position buf in
-      source#get buf;
-      let b = AFrame.pcm buf in
-      let position = AFrame.position buf in
+    method private generate_frame =
+      let c = source#get_mutable_content Frame.Fields.audio in
+      let b = Content.Audio.get_data c in
+      let position = source#frame_audio_position in
       let freq = freq () in
       let q = q () in
       let wet = wet () in
       let f = 2. *. sin (Float.pi *. freq /. rate) in
       for c = 0 to Array.length b - 1 do
         let b_c = b.(c) in
-        for i = offset to position - 1 do
+        for i = 0 to position - 1 do
           low.(c) <- low.(c) +. (f *. band.(c));
           high.(c) <- (q *. b_c.(i)) -. low.(c) -. (q *. band.(c));
           band.(c) <- (f *. high.(c)) +. band.(c);
@@ -82,7 +81,8 @@ class filter (source : source) freq q wet mode =
               | Notch -> notch.(c))
             +. ((1. -. wet) *. b_c.(i))
         done
-      done
+      done;
+      source#set_frame_data Frame.Fields.audio Content.Audio.lift_data b
   end
 
 let filter =
