@@ -88,18 +88,25 @@ class switch ~all_predicates ~override_meta ~transition_length ~replay_meta
     val mutable transition_length = transition_length
     val mutable selected : selection option = None
 
+    method set_selected v =
+      (match v with
+        | Some { child; effective_source } when effective_source != child.source
+          ->
+            effective_source#wake_up (self :> Clock.source)
+        | _ -> ());
+      (match selected with
+        | Some { child; effective_source } when effective_source != child.source
+          ->
+            effective_source#sleep (self :> Clock.source)
+        | _ -> ());
+      selected <- v
+
     (* We cannot reselect the same source twice during a streaming cycle. *)
     val mutable excluded_sources = []
 
     initializer
       self#on_before_streaming_cycle (fun () -> excluded_sources <- [])
 
-    method private is_selected_generated =
-      match selected with
-        | None -> false
-        | Some { child = { source }; effective_source } ->
-            source != effective_source
-
     method private select ~reselect () =
       let may_select ~single s =
         match selected with
@@ -123,6 +130,11 @@ class switch ~all_predicates ~override_meta ~transition_length ~replay_meta
              (not s.source#fallible) && (not single) && trivially_true d)
            children)
 
+    method private replay_meta m source =
+      let new_source = new Replay_metadata.replay m source in
+      Typing.(new_source#frame_type <: self#frame_type);
+      new_source
+
     method get_source ~reselect () =
       match selected with
         | Some s
@@ -145,7 +157,7 @@ class switch ~all_predicates ~override_meta ~transition_length ~replay_meta
                     () )
               with
                 | None, None -> ()
-                | Some _, None -> selected <- None
+                | Some _, None -> self#set_selected None
                 | None, Some (predicate, c) ->
                     self#log#important "Switch to %s." c.source#id;
                     let new_source =
@@ -157,15 +169,13 @@ class switch ~all_predicates ~override_meta ~transition_length ~replay_meta
                        * A quicker hack might have been doable if there wasn't a
                        * transition in between. *)
                         match c.source#last_metadata with
-                        | Some m when replay_meta ->
-                            new Insert_metadata.replay m c.source
+                        | Some (_, m) when replay_meta ->
+                            self#replay_meta m c.source
                         | _ -> c.source
                     in
-                    Typing.(new_source#frame_type <: self#frame_type);
-                    new_source#wake_up;
-                    selected <-
-                      Some
-                        { predicate; child = c; effective_source = new_source }
+                    self#set_selected
+                      (Some
+                         { predicate; child = c; effective_source = new_source })
                 | Some old_selection, Some (_, c)
                   when old_selection.child.source == c.source ->
                     ()
@@ -188,12 +198,10 @@ class switch ~all_predicates ~override_meta ~transition_length ~replay_meta
                        * A quicker hack might have been doable if there wasn't a
                        * transition in between. *)
                         match c.source#last_metadata with
-                        | Some m when replay_meta ->
-                            new Insert_metadata.replay m c.source
+                        | Some (_, m) when replay_meta ->
+                            self#replay_meta m c.source
                         | _ -> c.source
                     in
-                    Typing.(old_source#frame_type <: self#frame_type);
-                    Typing.(new_source#frame_type <: self#frame_type);
                     let s =
                       Lang.to_source
                         (Lang.apply c.transition
@@ -202,24 +210,20 @@ class switch ~all_predicates ~override_meta ~transition_length ~replay_meta
                              ("", Lang.source new_source);
                            ])
                     in
-                    Typing.(s#frame_type <: self#frame_type);
                     let s =
-                      match s#id with
-                        | id when id = new_source#id -> s
-                        | _ ->
-                            let s =
-                              new Max_duration.max_duration
-                                ~override_meta ~duration:transition_length s
-                            in
-                            Typing.(s#frame_type <: self#frame_type);
-                            (new Sequence.sequence ~merge:true [s; new_source]
-                              :> Source.source)
+                      if s == new_source then s
+                      else (
+                        Typing.(s#frame_type <: self#frame_type);
+                        let s =
+                          new Max_duration.max_duration
+                            ~override_meta ~duration:transition_length s
+                        in
+                        Typing.(s#frame_type <: self#frame_type);
+                        (new Sequence.sequence ~merge:true [s; new_source]
+                          :> Source.source))
                     in
-                    Typing.(s#frame_type <: self#frame_type);
-                    Clock.unify ~pos:self#pos s#clock self#clock;
-                    s#wake_up;
-                    selected <-
-                      Some { predicate; child = c; effective_source = s }
+                    self#set_selected
+                      (Some { predicate; child = c; effective_source = s })
             end;
             match selected with
               | Some s when s.effective_source#is_ready ->
@@ -263,14 +267,17 @@ let _ =
        true."
     ~meth:
       [
-        ( "selected",
-          ([], Lang.fun_t [] (Lang.nullable_t Lang.(source_t return_t))),
-          "Currently selected source.",
-          fun s ->
-            Lang.val_fun [] (fun _ ->
-                match s#selected with
-                  | Some s -> Lang.source s
-                  | None -> Lang.null) );
+        {
+          name = "selected";
+          scheme = ([], Lang.fun_t [] (Lang.nullable_t Lang.(source_t return_t)));
+          descr = "Currently selected source.";
+          value =
+            (fun s ->
+              Lang.val_fun [] (fun _ ->
+                  match s#selected with
+                    | Some s -> Lang.source s
+                    | None -> Lang.null));
+        };
       ]
     [
       ( "track_sensitive",
