@@ -136,28 +136,6 @@ let proto frame_t =
         Lang.int_t,
         Some (Lang.int Utils.pagesize),
         Some "Send data to clients using chunks of at least this length." );
-      ( "on_connect",
-        Lang.fun_t
-          [
-            (false, "headers", Lang.metadata_t);
-            (false, "uri", Lang.string_t);
-            (false, "protocol", Lang.string_t);
-            (false, "", Lang.string_t);
-          ]
-          Lang.unit_t,
-        Some
-          (Lang.val_cst_fun
-             [("headers", None); ("uri", None); ("protocol", None); ("", None)]
-             Lang.unit),
-        Some
-          "Callback executed when connection is established (takes headers, \
-           connection uri, protocol and client's IP as arguments)." );
-      ( "on_disconnect",
-        Lang.fun_t [(false, "", Lang.string_t)] Lang.unit_t,
-        Some (Lang.val_cst_fun [("", None)] Lang.unit),
-        Some
-          "Callback executed when connection stops (takes client's IP as \
-           argument)." );
       ( "headers",
         Lang.metadata_t,
         Some (Lang.list []),
@@ -303,21 +281,6 @@ class output p =
   let pos = Lang.pos p in
   let e f v = f (List.assoc v p) in
   let s v = e Lang.to_string v in
-  let on_connect = List.assoc "on_connect" p in
-  let on_disconnect = List.assoc "on_disconnect" p in
-  let on_connect ~headers ~protocol ~uri s =
-    ignore
-      (Lang.apply on_connect
-         [
-           ("headers", Lang.metadata headers);
-           ("uri", Lang.string uri);
-           ("protocol", Lang.string protocol);
-           ("", Lang.string s);
-         ])
-  in
-  let on_disconnect s =
-    ignore (Lang.apply on_disconnect [("", Lang.string s)])
-  in
   let metaint = Lang.to_int (List.assoc "metaint" p) in
   let data = encoder_data p in
   let encoding = Lang.to_string (List.assoc "encoding" p) in
@@ -360,14 +323,6 @@ class output p =
   let autostart = Lang.to_bool (List.assoc "start" p) in
   let infallible = not (Lang.to_bool (List.assoc "fallible" p)) in
   let register_telnet = Lang.to_bool (List.assoc "register_telnet" p) in
-  let on_start =
-    let f = List.assoc "on_start" p in
-    fun () -> ignore (Lang.apply f [])
-  in
-  let on_stop =
-    let f = List.assoc "on_stop" p in
-    fun () -> ignore (Lang.apply f [])
-  in
   let url = List.assoc "url" p |> Lang.to_option |> Option.map Lang.to_string in
   let port = e Lang.to_int "port" in
   let transport = e Lang.to_http_transport "transport" in
@@ -420,7 +375,7 @@ class output p =
     inherit
       [Strings.t] Output.encoded
         ~output_kind:"output.harbor" ~infallible ~register_telnet ~autostart
-          ~export_cover_metadata:false ~on_start ~on_stop ~name:mount source_val
+          ~export_cover_metadata:false ~name:mount source_val
 
     val mutable dump = None
     val mutable encoder = None
@@ -433,14 +388,18 @@ class output p =
     val metadata = { metadata = None; metadata_m = Mutex.create () }
     method encode frame = (Option.get encoder).Encoder.encode frame
     method self_sync = source#self_sync
+    val mutable on_connect = []
+    method on_connect fn = on_connect <- on_connect @ [fn]
+    val mutable on_disconnect = []
+    method on_disconnect fn = on_disconnect <- on_disconnect @ [fn]
 
-    method insert_metadata m =
+    method encode_metadata m =
       let m = Frame.Metadata.Export.to_metadata m in
       let m = recode m in
       Mutex_utils.mutexify metadata.metadata_m
         (fun () -> metadata.metadata <- Some m)
         ();
-      (Option.get encoder).Encoder.insert_metadata
+      (Option.get encoder).Encoder.encode_metadata
         (Frame.Metadata.Export.from_metadata ~cover:false m)
 
     method add_client ~protocol ~headers ~uri ~query s =
@@ -494,15 +453,19 @@ class output p =
           on_error =
             (fun e ->
               let bt = Printexc.get_backtrace () in
-              let msg =
+              let msg, bt =
                 match e with
-                  | Duppy.Io.Timeout -> Printf.sprintf "Timeout error for %s" ip
-                  | Duppy.Io.Io_error -> Printf.sprintf "I/O error for %s" ip
-                  | Duppy.Io.Unix (c, p, m) ->
-                      Printf.sprintf "Unix error for %s: %s" ip
-                        (Printexc.to_string (Unix.Unix_error (c, p, m)))
-                  | Duppy.Io.Unknown e ->
-                      Printf.sprintf "%s" (Printexc.to_string e)
+                  | Duppy.Io.Timeout ->
+                      (Printf.sprintf "Timeout error for %s" ip, bt)
+                  | Duppy.Io.Io_error ->
+                      (Printf.sprintf "I/O error for %s" ip, bt)
+                  | Duppy.Io.Unix (c, p, m, bt) ->
+                      ( Printf.sprintf "Unix error for %s: %s" ip
+                          (Printexc.to_string (Unix.Unix_error (c, p, m))),
+                        Printexc.raw_backtrace_to_string bt )
+                  | Duppy.Io.Unknown (e, bt) ->
+                      ( Printf.sprintf "%s" (Printexc.to_string e),
+                        Printexc.raw_backtrace_to_string bt )
               in
               Utils.log_exception ~log:self#log ~bt msg;
               self#log#info "Client %s disconnected" ip;
@@ -511,7 +474,7 @@ class output p =
                   client.state <- Done;
                   ignore (Strings.Mutable.flush client.buffer))
                 ();
-              on_disconnect ip;
+              List.iter (fun fn -> fn ip) on_disconnect;
               Harbor.Close (Harbor.mk_simple ""));
         }
       in
@@ -540,9 +503,7 @@ class output p =
              Mutex_utils.mutexify clients_m
                (fun () -> Queue.push client clients)
                ();
-             on_connect ~protocol ~uri
-               ~headers:(Frame.Metadata.from_list headers)
-               ip))
+             List.iter (fun fn -> fn ~headers ~uri ~protocol ip) on_connect))
 
     method send b =
       let slen = Strings.length b in
@@ -637,5 +598,56 @@ let _ =
   let return_t = Lang.frame_t (Lang.univ_t ()) Frame.Fields.empty in
   Lang.add_operator ~category:`Output
     ~descr:"Encode and output the stream using the harbor server."
-    ~meth:Output.meth ~base:Modules.output "harbor" (proto return_t) ~return_t
-    (fun p -> (new output p :> Output.output))
+    ~callbacks:
+      ([
+         {
+           Lang_source.name = "on_connect";
+           params = [];
+           descr =
+             "when connection is established (takes headers, connection uri, \
+              protocol and client's IP as arguments).";
+           register_deprecated_argument = true;
+           arg_t =
+             [
+               ( false,
+                 "",
+                 Lang.record_t
+                   [
+                     ("headers", Lang.metadata_t);
+                     ("uri", Lang.string_t);
+                     ("protocol", Lang.string_t);
+                     ("ip", Lang.string_t);
+                   ] );
+             ];
+           register =
+             (fun ~params:_ s on_connect ->
+               let on_connect ~headers ~uri ~protocol ip =
+                 on_connect
+                   [
+                     ( "",
+                       Lang.record
+                         [
+                           ("headers", Lang.metadata_list headers);
+                           ("uri", Lang.string uri);
+                           ("protocol", Lang.string protocol);
+                           ("ip", Lang.string ip);
+                         ] );
+                   ]
+               in
+               s#on_connect on_connect);
+         };
+         {
+           name = "on_disconnect";
+           params = [];
+           descr = "when a source is disconnected.";
+           register_deprecated_argument = true;
+           arg_t = [(false, "", Lang.string_t)];
+           register =
+             (fun ~params:_ s f ->
+               s#on_disconnect (fun ip -> f [("", Lang.string ip)]));
+         };
+       ]
+      @ Start_stop.callbacks ~label:"output")
+    ~meth:(Start_stop.meth ()) ~base:Modules.output "harbor" (proto return_t)
+    ~return_t
+    (fun p -> new output p)
