@@ -41,8 +41,6 @@ let proto =
            stopped until the source is available again." );
     ]
 
-let meth = Start_stop.meth ()
-
 module Queue = Queues.Queue
 
 (** Given abstract start stop and send methods, creates an output. Takes care of
@@ -50,8 +48,7 @@ module Queue = Queues.Queue
     last ten metadata and setups standard Server commands, including start/stop.
 *)
 class virtual output ~output_kind ?clock ?(name = "") ~infallible
-  ~register_telnet ~(on_start : unit -> unit) ~(on_stop : unit -> unit)
-  val_source autostart =
+  ~register_telnet val_source autostart =
   let source = Lang.to_source val_source in
   object (self)
     initializer
@@ -62,7 +59,7 @@ class virtual output ~output_kind ?clock ?(name = "") ~infallible
 
     initializer Typing.(source#frame_type <: self#frame_type)
     inherit active_operator ?clock ~name:output_kind [source]
-    inherit Start_stop.base ~on_start ~on_stop as start_stop
+    inherit Start_stop.base as start_stop
     method virtual private start : unit
     method virtual private stop : unit
     method virtual private send_frame : Frame.t -> unit
@@ -86,23 +83,23 @@ class virtual output ~output_kind ?clock ?(name = "") ~infallible
       if Queue.length metadata_queue > q_length then
         ignore (Queue.pop metadata_queue)
 
-    initializer self#on_metadata self#add_metadata
-
-    (* Registration of Telnet commands must be delayed because some operators
-       change their id at initialization time. *)
-    val mutable registered_telnet = false
-
-    method private register_telnet =
-      if register_telnet && not registered_telnet then (
-        registered_telnet <- true;
-        (* Add a few more server controls *)
-        let ns = [self#id] in
-        Server.add ~ns "skip"
+    initializer
+      self#on_frame (`Metadata self#add_metadata);
+      if register_telnet then (
+        self#register_command "skip"
           (fun _ ->
             self#skip;
             "Done")
           ~descr:"Skip current song.";
-        Server.add ~ns "metadata" ~descr:"Print current metadata." (fun _ ->
+        self#register_command "seek"
+          (fun v ->
+            try
+              let len = self#seek (Frame.main_of_seconds (float_of_string v)) in
+              Printf.sprintf "Seeked %.02f" (Frame.seconds_of_main len)
+            with exn -> Printf.sprintf "Error: %s" (Printexc.to_string exn))
+          ~usage:"seek <seconds>" ~descr:"Seek forward.";
+        self#register_command "metadata" ~descr:"Print current metadata."
+          (fun _ ->
             fst
               (Queue.fold metadata_queue
                  (fun m (s, i) ->
@@ -115,21 +112,14 @@ class virtual output ~output_kind ?clock ?(name = "") ~infallible
                    in
                    (s, i - 1))
                  ("", Queue.length metadata_queue)));
-        Server.add ~ns "remaining" ~descr:"Display estimated remaining time."
-          (fun _ ->
+        self#register_command "remaining"
+          ~descr:"Display estimated remaining time." (fun _ ->
             let r = source#remaining in
             if r < 0 then "(undef)"
             else (
               let t = Frame.seconds_of_main r in
               Printf.sprintf "%.2f" t)))
 
-    method private cleanup_telnet =
-      if registered_telnet then
-        List.iter
-          (Server.remove ~ns:[self#id])
-          ["skip"; "metadata"; "remaining"];
-      registered_telnet <- false
-
     method private can_generate_frame = source#is_ready
     method remaining = source#remaining
     method abort_track = source#abort_track
@@ -151,12 +141,8 @@ class virtual output ~output_kind ?clock ?(name = "") ~infallible
                   use an expliciy type annotation!"
                  self#id);
 
-          if not autostart then start_stop#transition_to `Stopped;
-
-          self#register_telnet);
-      self#on_sleep (fun () ->
-          self#cleanup_telnet;
-          start_stop#transition_to `Stopped)
+          if not autostart then start_stop#transition_to `Stopped);
+      self#on_sleep (fun () -> start_stop#transition_to `Stopped)
 
     (* The output process *)
     val mutable skip = false
@@ -188,14 +174,13 @@ class virtual output ~output_kind ?clock ?(name = "") ~infallible
           self#abort_track))
   end
 
-class dummy ?clock ~infallible ~on_start ~on_stop ~autostart ~register_telnet
-  source =
+class dummy ?clock ~infallible ~autostart ~register_telnet source =
   let s = Lang.to_source source in
   object
     inherit
       output
         source autostart ?clock ~name:"dummy" ~output_kind:"output.dummy"
-          ~infallible ~on_start ~on_stop ~register_telnet
+          ~infallible ~register_telnet
 
     method! private reset = ()
     method private start = ()
@@ -209,31 +194,26 @@ let _ =
   Lang.add_operator ~base:Modules.output "dummy"
     (proto @ [("", Lang.source_t return_t, None, None)])
     ~category:`Output
-    ~descr:"Dummy output: computes the stream, without actually using it." ~meth
+    ~descr:"Dummy output: computes the stream, without actually using it."
+    ~meth:(Start_stop.meth ())
+    ~callbacks:(Start_stop.callbacks ~label:"output")
     ~return_t
     (fun p ->
       let infallible = not (Lang.to_bool (List.assoc "fallible" p)) in
       let autostart = Lang.to_bool (List.assoc "start" p) in
-      let on_start = List.assoc "on_start" p in
-      let on_stop = List.assoc "on_stop" p in
-      let on_start () = ignore (Lang.apply on_start []) in
-      let on_stop () = ignore (Lang.apply on_stop []) in
       let register_telnet = Lang.to_bool (List.assoc "register_telnet" p) in
-      new dummy
-        ~on_start ~on_stop ~infallible ~autostart ~register_telnet
-        (List.assoc "" p))
+      new dummy ~infallible ~autostart ~register_telnet (List.assoc "" p))
 
 (** More concrete abstract-class, which takes care of the #send_frame method for
     outputs based on encoders. *)
-class virtual ['a] encoded ~output_kind ?clock ~name ~infallible ~on_start
-  ~on_stop ~register_telnet ~autostart ~export_cover_metadata source =
+class virtual ['a] encoded ~output_kind ?clock ~name ~infallible
+  ~register_telnet ~autostart ~export_cover_metadata source =
   object (self)
     inherit
       output
-        ~infallible ~on_start ~on_stop ~output_kind ?clock ~name
-          ~register_telnet source autostart
+        ~infallible ~output_kind ?clock ~name ~register_telnet source autostart
 
-    method virtual private insert_metadata : Frame.Metadata.Export.t -> unit
+    method virtual private encode_metadata : Frame.Metadata.Export.t -> unit
     method virtual private encode : Frame.t -> 'a
     method virtual private send : 'a -> unit
 
@@ -245,7 +225,7 @@ class virtual ['a] encoded ~output_kind ?clock ~name ~infallible ~on_start
           match Frame.get_metadata frame start with
             | None -> ()
             | Some m ->
-                self#insert_metadata
+                self#encode_metadata
                   (Frame.Metadata.Export.from_metadata
                      ~cover:export_cover_metadata m)
         in
