@@ -33,8 +33,6 @@ class virtual output :
   ?name:string ->
   infallible:bool ->
   register_telnet:bool ->
-  on_start:(unit -> unit) ->
-  on_stop:(unit -> unit) ->
   Lang.value ->
   bool ->
 object
@@ -48,6 +46,8 @@ object
   method transition_to : Start_stop.state -> unit
   method seek_source : Source.source
   method output : unit
+  method on_start : (unit -> unit) -> unit
+  method on_stop : (unit -> unit) -> unit
   method private video_dimensions : int * int
   method private reset : unit
   method virtual private send_frame : Frame.t -> unit
@@ -55,16 +55,11 @@ object
   method virtual private stop : unit
 end
 
-(** Default methods on output values. *)
-val meth : (string * Lang.scheme * string * (output -> Lang.value)) list
-
 class virtual ['a] encoded :
   output_kind:string ->
   ?clock:Clock.t ->
   name:string ->
   infallible:bool ->
-  on_start:(unit -> unit) ->
-  on_stop:(unit -> unit) ->
   register_telnet:bool ->
   autostart:bool ->
   export_cover_metadata:bool ->
@@ -73,7 +68,7 @@ object
   inherit output
   method private send_frame : Frame.t -> unit
   method virtual private encode : Frame.t -> 'a
-  method virtual private insert_metadata : Frame.Metadata.Export.t -> unit
+  method virtual private encode_metadata : Frame.Metadata.Export.t -> unit
   method virtual private send : 'a -> unit
   method private reset : unit
   method virtual private start : unit
@@ -83,8 +78,6 @@ end
 class dummy :
   ?clock:Clock.t ->
   infallible:bool ->
-  on_start:(unit -> unit) ->
-  on_stop:(unit -> unit) ->
   autostart:bool ->
   register_telnet:bool ->
   Lang.value ->
