@@ -45,22 +45,14 @@ class virtual base ?clock ~source:source_val ~name p =
   let autostart = e Lang.to_bool "start" in
   let infallible = not (Lang.to_bool (List.assoc "fallible" p)) in
   let register_telnet = Lang.to_bool (List.assoc "register_telnet" p) in
-  let on_start =
-    let f = List.assoc "on_start" p in
-    fun () -> ignore (Lang.apply f [])
-  in
-  let on_stop =
-    let f = List.assoc "on_stop" p in
-    fun () -> ignore (Lang.apply f [])
-  in
   let export_cover_metadata =
     Lang.to_bool (List.assoc "export_cover_metadata" p)
   in
   object (self)
     inherit
       [Strings.t] Output.encoded
-        ?clock ~infallible ~register_telnet ~on_start ~on_stop ~autostart
-          ~export_cover_metadata ~output_kind:"output.file" ~name source_val
+        ?clock ~infallible ~register_telnet ~autostart ~export_cover_metadata
+          ~output_kind:"output.file" ~name source_val
 
     val mutable encoder = None
     val mutable current_metadata = None
@@ -99,10 +91,10 @@ class virtual base ?clock ~source:source_val ~name p =
     method virtual write_pipe : string -> int -> int -> unit
     method send b = Strings.iter self#write_pipe b
 
-    method insert_metadata m =
+    method encode_metadata m =
       match encoder with
         | None -> ()
-        | Some encoder -> encoder.Encoder.insert_metadata m
+        | Some encoder -> encoder.Encoder.encode_metadata m
   end
 
 (** url output: discard encoded data, try to restart on encoding error (can be
@@ -115,10 +107,6 @@ let url_proto frame_t =
         Lang.nullable_t Lang.float_t,
         Some (Lang.float 2.),
         Some "If not `null`, restart output on errors after the given delay." );
-      ( "on_error",
-        Lang.fun_t [(false, "", Lang.error_t)] Lang.unit_t,
-        Some (Lang.val_fun [("", "", None)] (fun _ -> Lang.unit)),
-        Some "Callback executed when an error occurs." );
       ( "self_sync",
         Lang.bool_t,
         Some (Lang.bool false),
@@ -129,6 +117,25 @@ let url_proto frame_t =
       ("", Lang.source_t frame_t, None, None);
     ]
 
+let url_callbacks =
+  Lang.
+    [
+      {
+        name = "on_error";
+        params = [];
+        descr = "when an error occurs.";
+        register_deprecated_argument = true;
+        arg_t = [(false, "", Lang.error_t)];
+        register =
+          (fun ~params:_ s f ->
+            s#on_error (fun ~bt exn ->
+                let error =
+                  Lang.runtime_error_of_exception ~bt ~kind:"output" exn
+                in
+                f [("", Lang.error error)]));
+      };
+    ]
+
 class url_output p =
   let url = Lang.to_string (List.assoc "url" p) in
   let format_val = Lang.assoc "" 1 p in
@@ -144,23 +151,6 @@ class url_output p =
   let restart_delay =
     Lang.to_valued_option Lang.to_float (List.assoc "restart_delay" p)
   in
-  let on_error = List.assoc "on_error" p in
-  let on_error ~bt exn =
-    let error = Lang.runtime_error_of_exception ~bt ~kind:"output" exn in
-    ignore (Lang.apply on_error [("", Lang.error error)])
-  in
-  let on_start =
-    let f = List.assoc "on_start" p in
-    fun () -> ignore (Lang.apply f [])
-  in
-  let p =
-    List.map
-      (fun ((lbl, _) as v) ->
-        match lbl with
-          | "on_start" -> ("on_start", Lang.val_fun [] (fun _ -> Lang.unit))
-          | _ -> v)
-      p
-  in
   let self_sync = Lang.to_bool (List.assoc "self_sync" p) in
   let name = "output.url" in
   object (self)
@@ -168,28 +158,33 @@ class url_output p =
     method private encoder_factory = encoder_factory ~format format_val
     val mutable restart_time = 0.
     method can_connect = restart_time <= Unix.gettimeofday ()
+    val mutable on_error = []
+    method on_error fn = on_error <- on_error @ [fn]
 
-    method on_error ~bt exn =
+    method apply_on_error ~bt exn =
       (try ignore self#close_encoder with _ -> ());
       Utils.log_exception ~log:self#log
         ~bt:(Printexc.raw_backtrace_to_string bt)
         (Printf.sprintf "Error while connecting: %s" (Printexc.to_string exn));
-      on_error ~bt exn;
+      List.iter (fun fn -> fn ~bt exn) on_error;
       match restart_delay with
         | None -> Printexc.raise_with_backtrace exn bt
         | Some delay ->
             restart_time <- Unix.gettimeofday () +. delay;
             self#log#important "Will try again in %.02f seconds." delay
 
+    val mutable on_start = []
+    method! on_start fn = on_start <- fn :: on_start
+
     method connect =
       match encoder with
         | None when self#can_connect -> (
             try
               self#create_encoder;
-              on_start ()
+              List.iter (fun fn -> fn ()) on_start
             with exn ->
               let bt = Printexc.get_raw_backtrace () in
-              self#on_error ~bt exn)
+              self#apply_on_error ~bt exn)
         | _ -> ()
 
     method start = self#connect
@@ -205,7 +200,7 @@ class url_output p =
           | Some _ -> base#encode frame
       with exn ->
         let bt = Printexc.get_raw_backtrace () in
-        self#on_error ~bt exn;
+        self#apply_on_error ~bt exn;
         Strings.empty
 
     method write_pipe _ _ _ = ()
@@ -215,18 +210,19 @@ class url_output p =
 let _ =
   let return_t = Lang.univ_t () in
   Lang.add_operator ~base:output "url" (url_proto return_t) ~return_t
-    ~category:`Output ~meth:Output.meth
+    ~category:`Output ~meth:(Start_stop.meth ())
+    ~callbacks:(url_callbacks @ Start_stop.callbacks ~label:"output")
     ~descr:
       "Encode and let encoder handle data output. Useful with encoder with no \
        expected output or to encode to files that need full control from the \
-       encoder, e.g. `%ffmpeg` with `rtmp` output." (fun p ->
-      (new url_output p :> Output.output))
+       encoder, e.g. `%ffmpeg` with `rtmp` output."
+    (fun p -> new url_output p)
 
 (** Piped virtual class: open/close pipe, implements metadata interpolation and
     takes care of the various reload mechanisms. *)
 
 let default_reopen_on_error =
-  Lang.eval ~cache:false ~stdlib:`Disabled ~typecheck:false "fun (_) -> null()"
+  Lang.eval ~cache:false ~stdlib:`Disabled ~typecheck:false "fun (_) -> null"
 
 let default_reopen_on_metadata =
   Lang.eval ~cache:false ~stdlib:`Disabled ~typecheck:false "fun (_) -> false"
@@ -264,30 +260,38 @@ let pipe_proto frame_t arg_doc =
         Some
           "Prevent re-opening within that delay, in seconds. Only applies to \
            `reopen_when`." );
-      ( "on_reopen",
-        Lang.fun_t [] Lang.unit_t,
-        Some (Lang.val_cst_fun [] Lang.unit),
-        Some "Callback executed when the output is reopened." );
       ("", Lang.format_t frame_t, None, Some "Encoding format.");
       ("", Lang.getter_t Lang.string_t, None, Some arg_doc);
       ("", Lang.source_t frame_t, None, None);
     ]
 
+let pipe_callbacks =
+  Lang.
+    [
+      {
+        name = "on_reopen";
+        params = [];
+        descr = "when the output is reopened.";
+        register_deprecated_argument = true;
+        arg_t = [];
+        register = (fun ~params:_ s f -> s#on_reopen (fun () -> f []));
+      };
+    ]
+
 let pipe_meth =
-  let meth =
-    List.map
-      (fun (a, b, c, fn) -> (a, b, c, fun s -> fn (s :> Output.output)))
-      Output.meth
-  in
-  ( "reopen",
-    ([], Lang.fun_t [] Lang.unit_t),
-    "Reopen the output pipe. The actual reopening happens the next time the \
-     output has some data to output.",
-    fun s ->
-      Lang.val_fun [] (fun _ ->
-          s#need_reopen;
-          Lang.unit) )
-  :: meth
+  {
+    Lang.name = "reopen";
+    scheme = ([], Lang.fun_t [] Lang.unit_t);
+    descr =
+      "Reopen the output pipe. The actual reopening happens the next time the \
+       output has some data to output.";
+    value =
+      (fun s ->
+        Lang.val_fun [] (fun _ ->
+            s#need_reopen;
+            Lang.unit));
+  }
+  :: Start_stop.meth ()
 
 class virtual piped_output ?clock ~name p =
   let source = Lang.assoc "" 3 p in
@@ -309,8 +313,6 @@ class virtual piped_output ?clock ~name p =
   let reopen_when = List.assoc "reopen_when" p in
   let reopen_when () = Lang.to_bool (Lang.apply reopen_when []) in
   let reopen_delay = Lang.to_float_getter (List.assoc "reopen_delay" p) in
-  let on_reopen = List.assoc "on_reopen" p in
-  let on_reopen () = ignore (Lang.apply on_reopen []) in
   object (self)
     inherit base ?clock ~source ~name p as base
     val mutable open_date = 0.
@@ -341,6 +343,8 @@ class virtual piped_output ?clock ~name p =
         base#send self#close_encoder;
         self#close_pipe)
 
+    val mutable on_reopen = []
+    method on_reopen fn = on_reopen <- on_reopen @ [fn]
     method start = self#prepare_pipe
     method stop = self#cleanup_pipe
 
@@ -348,7 +352,7 @@ class virtual piped_output ?clock ~name p =
       self#log#important "Re-opening output pipe.";
       self#cleanup_pipe;
       self#prepare_pipe;
-      on_reopen ()
+      List.iter (fun fn -> fn ()) on_reopen
 
     method private reopen_on_error ~bt exn =
       let error = Lang.runtime_error_of_exception ~bt ~kind:"output" exn in
@@ -382,10 +386,10 @@ class virtual piped_output ?clock ~name p =
         | _ -> ());
       base#encode frame
 
-    method! insert_metadata metadata =
+    method! encode_metadata metadata =
       if reopen_on_metadata (Frame.Metadata.Export.to_metadata metadata) then
         self#reopen;
-      base#insert_metadata metadata
+      base#encode_metadata metadata
   end
 
 (** Out channel virtual class: takes care of current out channel and writing to
@@ -462,7 +466,7 @@ class virtual ['a] file_output_base p =
         | None -> (
             let filename = self#filename in
             try
-              Utils.mkdir ~perm:dir_perm (Filename.dirname filename);
+              Utils.ensure_dir ~perm:dir_perm filename;
               Atomic.set current_filename (Some filename);
               (filename, mode, perm)
             with Sys_error _ as exn ->
@@ -587,7 +591,9 @@ let output_file =
   let return_t = Lang.univ_t () in
   Lang.add_operator ~base:output "file" (file_proto return_t) ~return_t
     ~category:`Output ~meth:pipe_meth
-    ~descr:"Output the source stream to a file." (fun p -> new_file_output p)
+    ~callbacks:(pipe_callbacks @ Start_stop.callbacks ~label:"output")
+    ~descr:"Output the source stream to a file."
+    (fun p -> new_file_output p)
 
 (** External output *)
 
@@ -628,6 +634,6 @@ let _ =
   let return_t = Lang.univ_t () in
   Lang.add_operator ~base:output "external"
     (pipe_proto return_t "Process to pipe data to.")
-    ~return_t ~category:`Output ~meth:pipe_meth
+    ~return_t ~category:`Output ~meth:pipe_meth ~callbacks:pipe_callbacks
     ~descr:"Send the stream to a process' standard input." (fun p ->
       (new external_output p :> piped_output))
