@@ -65,16 +65,24 @@ type watcher = {
   after_streaming_cycle : unit -> unit;
 }
 
-let source_log = Log.make ["source"]
+type position_callback = {
+  mode : [ `Remaining | `Elapsed ];
+  allow_partial : bool;
+  position : unit -> int;
+  on_position : pos:float -> Frame.metadata -> unit;
+  mutable executed : bool;
+}
 
-let finalise s =
-  source_log#debug "Source %s is collected." s#id;
-  try s#force_sleep
-  with e ->
-    let bt = Printexc.get_backtrace () in
-    Utils.log_exception ~log:source_log ~bt
-      (Printf.sprintf "Error when leaving output %s: %s!" s#id
-         (Printexc.to_string e))
+type frame_callback = { before : bool; on_frame : unit -> unit }
+
+type on_frame =
+  [ `Metadata of Frame.metadata -> unit
+  | `Track of Frame.metadata -> unit
+  | `Position of position_callback
+  | `Frame of frame_callback ]
+
+let source_log = Log.make ["source"]
+let finalise s = source_log#debug "Source %s is collected." s#id
 
 class virtual operator ?(stack = []) ?clock ~name sources =
   let frame_type = Type.var () in
@@ -106,7 +114,7 @@ class virtual operator ?(stack = []) ?clock ~name sources =
     val mutable log = source_log
     method private create_log = log <- Log.make [self#id]
     method log = log
-    val mutable id = Lang_string.generate_id name
+    val mutable id = Lang_string.generate_id ~category:"source" name
     method id = id
 
     method set_id ?(force = true) s =
@@ -117,7 +125,7 @@ class virtual operator ?(stack = []) ?clock ~name sources =
           s
       in
       if force && s <> self#id then (
-        id <- Lang_string.generate_id s;
+        id <- Lang_string.generate_id ~category:"source" s;
 
         (* Sometimes the ID is changed during initialization, in order to make it
          equal to the server name, which is only registered at initialization
@@ -191,8 +199,8 @@ class virtual operator ?(stack = []) ?clock ~name sources =
               failwith
                 (Printf.sprintf
                    "Early computation of source content-type detected for \
-                    source %s!"
-                   self#id);
+                    source %s on clock %s!"
+                   self#id (Clock.id self#clock));
             self#log#debug "Assigning source content type for frame type: %s"
               (Type.to_string self#frame_type);
             let ct = Frame_type.content_type self#frame_type in
@@ -246,11 +254,11 @@ class virtual operator ?(stack = []) ?clock ~name sources =
         | Some dim -> dim
 
     val mutable on_wake_up = []
-    method on_wake_up fn = on_wake_up <- fn :: on_wake_up
+    method on_wake_up fn = on_wake_up <- on_wake_up @ [fn]
 
     initializer
       self#on_wake_up (fun () ->
-          List.iter (fun s -> s#wake_up) sources;
+          List.iter (fun s -> s#wake_up (self :> Clock.source)) sources;
           self#iter_watchers (fun w ->
               w.wake_up ~fallible:self#fallible ~source_type:self#source_type
                 ~id:self#id ~ctype:self#content_type
@@ -259,8 +267,10 @@ class virtual operator ?(stack = []) ?clock ~name sources =
     val is_up : [ `False | `True | `Error ] Atomic.t = Atomic.make `False
     method is_up = Atomic.get is_up = `True
     val streaming_state : streaming_state Atomic.t = Atomic.make `Pending
+    val mutable activations : Clock.source list = []
 
-    method wake_up =
+    method wake_up (src : Clock.source) =
+      self#mutexify (fun () -> activations <- src :: activations) ();
       if Atomic.compare_and_set is_up `False `True then (
         try
           self#content_type_computation_allowed;
@@ -284,22 +294,39 @@ class virtual operator ?(stack = []) ?clock ~name sources =
           Printexc.raise_with_backtrace exn bt)
 
     val mutable on_sleep = []
-    method on_sleep fn = on_sleep <- fn :: on_sleep
+    method on_sleep fn = on_sleep <- on_sleep @ [fn]
 
-    method force_sleep =
+    method private actual_sleep =
       if Atomic.compare_and_set is_up `True `False then (
         source_log#info "Source %s gets down." self#id;
         List.iter (fun fn -> fn ()) on_sleep)
 
-    method sleep =
-      match (Clock.started self#clock, Atomic.get streaming_state) with
-        | true, (`Ready _ | `Unavailable) ->
-            Clock.after_tick self#clock (fun () -> self#force_sleep)
-        | _ -> self#force_sleep
+    method sleep (src : Clock.source) =
+      self#mutexify
+        (fun () ->
+          let found, l =
+            List.fold_left
+              (fun (found, l) s ->
+                if (not found) && s == src then (true, l) else (found, s :: l))
+              (false, []) activations
+          in
+          if not found then
+            self#log#critical "Not activations found for %s" src#id;
+          activations <- l)
+        ();
+      match
+        (activations, Clock.started self#clock, Atomic.get streaming_state)
+      with
+        | _ :: _, _, _ -> ()
+        | _, true, (`Ready _ | `Unavailable) ->
+            Clock.after_tick self#clock (fun () -> self#actual_sleep)
+        | _ -> self#actual_sleep
 
     initializer
       Gc.finalise finalise self;
-      self#on_sleep (fun () -> self#iter_watchers (fun w -> w.sleep ()))
+      self#on_sleep (fun () ->
+          List.iter (fun s -> s#sleep (self :> Clock.source)) sources;
+          self#iter_watchers (fun w -> w.sleep ()))
 
     (** Streaming *)
 
@@ -325,7 +352,7 @@ class virtual operator ?(stack = []) ?clock ~name sources =
     method virtual private generate_frame : Frame.t
 
     method is_ready =
-      if self#is_up then (
+      if self#is_up && Clock.started self#clock then (
         self#before_streaming_cycle;
         match Atomic.get streaming_state with
           | `Ready _ | `Done _ -> true
@@ -337,7 +364,7 @@ class virtual operator ?(stack = []) ?clock ~name sources =
     val mutable on_before_streaming_cycle = []
 
     method on_before_streaming_cycle fn =
-      on_before_streaming_cycle <- fn :: on_before_streaming_cycle
+      on_before_streaming_cycle <- on_before_streaming_cycle @ [fn]
 
     initializer
       self#on_before_streaming_cycle (fun () ->
@@ -346,7 +373,7 @@ class virtual operator ?(stack = []) ?clock ~name sources =
     val mutable on_after_streaming_cycle = []
 
     method on_after_streaming_cycle fn =
-      on_after_streaming_cycle <- fn :: on_after_streaming_cycle
+      on_after_streaming_cycle <- on_after_streaming_cycle @ [fn]
 
     initializer
       self#on_after_streaming_cycle (fun () ->
@@ -469,40 +496,44 @@ class virtual operator ?(stack = []) ?clock ~name sources =
             empty_frame <- Some f;
             f
 
+    val insert_metadata = Atomic.make None
+
+    method insert_metadata ~new_track m =
+      Atomic.set insert_metadata (Some (new_track, m))
+
     method end_of_track = Frame.add_track_mark self#empty_frame 0
     val mutable last_metadata = None
     method last_metadata = last_metadata
-    val mutable on_metadata : (Frame.metadata -> unit) List.t = []
-    method on_metadata fn = on_metadata <- fn :: on_metadata
-    val mutable on_track_called = false
-
-    initializer
-      self#on_before_streaming_cycle (fun () -> on_track_called <- false)
-
+    val mutable on_frame : on_frame list = []
+    method on_frame fn = on_frame <- on_frame @ [fn]
     val mutable reset_last_metadata_on_track = Atomic.make true
 
     method reset_last_metadata_on_track =
       Atomic.get reset_last_metadata_on_track
 
-    method set_reset_last_metadata_on_track =
-      Atomic.set reset_last_metadata_on_track
-
-    val mutable on_track : (Frame.metadata -> unit) List.t = []
-    method on_track fn = on_track <- fn :: on_track
+    method set_reset_last_metadata_on_track v =
+      Atomic.set reset_last_metadata_on_track v
 
     method private set_last_metadata buf =
       match List.rev (Frame.get_all_metadata buf) with
-        | (_, m) :: _ -> last_metadata <- Some m
+        | v :: _ -> last_metadata <- Some v
         | _ -> ()
 
+    val mutable on_track_called = false
+
+    initializer
+      self#on_before_streaming_cycle (fun () -> on_track_called <- false)
+
     method private execute_on_track buf =
       if not on_track_called then (
         on_track_called <- true;
         if self#reset_last_metadata_on_track then last_metadata <- None;
         self#set_last_metadata buf;
-        let m = Option.value ~default:Frame.Metadata.empty last_metadata in
+        let _, m =
+          Option.value ~default:(0, Frame.Metadata.empty) last_metadata
+        in
         self#log#debug "calling on_track handlers..";
-        List.iter (fun fn -> fn m) on_track)
+        List.iter (function `Track fn -> fn m | _ -> ()) on_frame)
 
     val mutable last_images = Hashtbl.create 0
 
@@ -572,33 +603,110 @@ class virtual operator ?(stack = []) ?clock ~name sources =
           then self#normalize_video ~field content
           else content)
 
+    method private on_position ~end_of_track buf =
+      let length = Frame.position buf in
+      elapsed <- elapsed + length;
+      let rem = self#remaining in
+      self#set_last_metadata buf;
+      let _, m =
+        Option.value ~default:(0, Frame.Metadata.empty) last_metadata
+      in
+      let is_allowed = function
+        | { executed = true } -> false
+        | { allow_partial = true } when end_of_track -> true
+        | { mode = `Elapsed; position } -> position () <= elapsed
+        | { mode = `Remaining; position } -> 0 <= rem && rem <= position ()
+      in
+      let position = function
+        | { mode = `Elapsed } -> Frame.seconds_of_main elapsed
+        | { mode = `Remaining } -> Frame.seconds_of_main rem
+      in
+      List.iter
+        (function
+          | `Position p when is_allowed p ->
+              p.executed <- true;
+              p.on_position ~pos:(position p) m
+          | _ -> ())
+        on_frame
+
     method private instrumented_generate_frame =
       let start_time = Unix.gettimeofday () in
+      let on_frame = self#mutexify (fun () -> on_frame) () in
+      List.iter
+        (function `Frame { before = true; on_frame } -> on_frame () | _ -> ())
+        on_frame;
       let buf = self#normalize_video_content self#generate_frame in
+      List.iter
+        (function
+          | `Frame { before = false; on_frame } -> on_frame () | _ -> ())
+        on_frame;
       let end_time = Unix.gettimeofday () in
       let length = Frame.position buf in
-      let track_marks = Frame.track_marks buf in
       let buf =
-        match track_marks with
-          | p :: _ :: _ ->
+        match (Atomic.exchange insert_metadata None, Frame.track_marks buf) with
+          | Some (new_track, m), _ ->
+              let m =
+                Frame.Metadata.append
+                  (Option.value ~default:Frame.Metadata.empty
+                     (Frame.get_metadata buf 0))
+                  m
+              in
+              let buf = Frame.add_metadata buf 0 m in
+              if new_track then Frame.(add_track_mark (drop_track_marks buf)) 0
+              else buf
+          | None, p :: _ :: _ ->
               self#log#important
                 "Source created multiple tracks in a single frame! Sub-frame \
                  tracks are not supported and are merged into a single one..";
               Frame.add_track_mark (Frame.drop_track_marks buf) p
           | _ -> buf
       in
-      let has_track_mark = track_marks <> [] in
-      if has_track_mark then elapsed <- 0 else elapsed <- elapsed + length;
+      let has_track_mark =
+        match self#split_frame buf with
+          | buf, Some new_track ->
+              if 0 < elapsed then self#on_position ~end_of_track:true buf;
+              elapsed <- 0;
+              if self#reset_last_metadata_on_track then last_metadata <- None;
+              List.iter
+                (function `Position p -> p.executed <- false | _ -> ())
+                on_frame;
+              self#on_position ~end_of_track:false new_track;
+              true
+          | buf, None ->
+              self#on_position ~end_of_track:false buf;
+              false
+      in
+
+      (* Executing track mark and metadata callbacks is tricky.
+         - We want to preserve the order with which they are registered.
+         - We want to execute track marks even without metadata.
+
+      First: execute both kind of callback using the metadata from the frame: *)
       let metadata = Frame.get_all_metadata buf in
-      let on_metadata = self#mutexify (fun () -> on_metadata) () in
       List.iter
-        (fun (i, m) ->
+        (fun (p, m) ->
           self#log#debug
-            "generate_frame: got metadata at position %d: calling handlers..." i;
-          List.iter (fun fn -> fn m) on_metadata)
+            "generate_frame: got metadata at position %d: calling handlers..." p;
+          let is_on_track =
+            match last_metadata with
+              | Some (p', _)
+                when (not on_track_called) && has_track_mark && p = p' ->
+                  self#log#debug "calling on_track handlers..";
+                  on_track_called <- true;
+                  true
+              | _ -> false
+          in
+          List.iter
+            (fun cb ->
+              match cb with
+                | `Metadata fn -> fn m
+                | `Track fn when is_on_track -> fn m
+                | _ -> ())
+            on_frame)
         metadata;
-      if has_track_mark then self#execute_on_track buf
-      else self#set_last_metadata buf;
+
+      (* Then, if we have a track mark but no metadata, executed the on_track callbacks. *)
+      if has_track_mark && not on_track_called then self#execute_on_track buf;
       self#iter_watchers (fun w ->
           w.generate_frame ~start_time ~end_time ~length ~has_track_mark
             ~metadata);
