@@ -74,6 +74,24 @@ type watcher = {
   after_streaming_cycle : unit -> unit;
 }
 
+(** Callbacks executed when computing frames. *)
+
+type position_callback = {
+  mode : [ `Remaining | `Elapsed ];
+  allow_partial : bool;
+  position : unit -> int;
+  on_position : pos:float -> Frame.metadata -> unit;
+  mutable executed : bool;
+}
+
+type frame_callback = { before : bool; on_frame : unit -> unit }
+
+type on_frame =
+  [ `Metadata of Frame.metadata -> unit
+  | `Track of Frame.metadata -> unit
+  | `Position of position_callback
+  | `Frame of frame_callback ]
+
 (** The [source] use is to send data frames through the [get] method. *)
 class virtual source :
   ?stack:Pos.t list ->
@@ -133,17 +151,14 @@ object
   method on_wake_up : (unit -> unit) -> unit
 
   (** Called when the source must be ready. Can be called multiple times *)
-  method wake_up : unit
+  method wake_up : Clock.source -> unit
 
   (** Register a callback when sleep is called. *)
   method on_sleep : (unit -> unit) -> unit
 
   (** Called when the source can release all its resources. Can be called
       concurrently and multiple times. *)
-  method sleep : unit
-
-  (** Force the source to sleep. Should be called by the clocks only. *)
-  method force_sleep : unit
+  method sleep : Clock.source -> unit
 
   (** Check if a source is up or not. *)
   method is_up : bool
@@ -212,7 +227,7 @@ object
   method seek : int -> int
 
   (** The source's last metadata. *)
-  method last_metadata : Frame.metadata option
+  method last_metadata : (int * Frame.metadata) option
 
   method reset_last_metadata_on_track : bool
   method set_reset_last_metadata_on_track : bool -> unit
@@ -222,12 +237,8 @@ object
   method register_command :
     ?usage:string -> descr:string -> string -> (string -> string) -> unit
 
-  (** Register a callback to be called on new metadata *)
-  method on_metadata : (Frame.metadata -> unit) -> unit
-
-  (** Register a callback to be called on new track. Callback is called with the
-      most recent metadata before a given track mark. *)
-  method on_track : (Frame.metadata -> unit) -> unit
+  (** Register a callback to be called when computing frames. *)
+  method on_frame : on_frame -> unit
 
   (** These two are used by [generate_from_multiple_sources] and should not be
       used otherwise. *)
@@ -235,6 +246,10 @@ object
 
   method private set_last_metadata : Frame.t -> unit
 
+  (** Insert a metadata at the beginning of the new frame. Also add a track mark
+      when [new_track] is [true] *)
+  method insert_metadata : new_track:bool -> Frame.metadata -> unit
+
   (** Sources must implement this method. It should return [true] when the
       source can produce data during the current streaming cycle. *)
   method virtual private can_generate_frame : bool
