@@ -1,7 +1,7 @@
 (*****************************************************************************
 
-  Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2023 Savonet team
+  Liquidsoap, a programmable stream generator.
+  Copyright 2003-2024 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -24,12 +24,12 @@ class fail name =
   object (self)
     inherit Source.source ~name ()
     method seek_source = (self :> Source.source)
-    method stype = `Fallible
-    method private _is_ready ?frame:_ _ = false
-    method self_sync = (`Static, false)
+    method fallible = true
+    method private can_generate_frame = false
+    method self_sync = (`Static, None)
     method remaining = 0
     method abort_track = ()
-    method get_frame _ = assert false
+    method generate_frame = assert false
   end
 
 let fail () = (new fail "fail" :> Source.source)
@@ -43,11 +43,13 @@ let fail =
     ~return_t [] (fun _ -> (new fail "source.fail" :> Source.source))
 
 class fail_init =
-  object
+  object (self)
     inherit fail "source.fail.init"
 
-    method! wake_up _ =
-      Lang.raise_error ~pos:[] ~message:"Source's initialization failed" "debug"
+    initializer
+      self#on_wake_up (fun () ->
+          Lang.raise_error ~pos:[] ~message:"Source's initialization failed"
+            "debug")
   end
 
 let _ =
@@ -56,4 +58,28 @@ let _ =
     ~descr:
       "A source that errors during its initialization phase, used for testing \
        and debugging." ~flags:[`Experimental] ~return_t [] (fun _ ->
-      (new fail_init :> Source.source))
+      new fail_init)
+
+class is_ready s =
+  object (self)
+    inherit Source.operator ~name:"is_ready" [s]
+    method seek_source = (self :> Source.source)
+    method fallible = true
+    method private can_generate_frame = true
+    method self_sync = (`Static, None)
+    method remaining = 0
+    method abort_track = ()
+    method generate_frame = if s#is_ready then s#get_frame else self#empty_frame
+  end
+
+let _ =
+  let return_t = Lang.frame_t (Lang.univ_t ()) Frame.Fields.empty in
+  Lang.add_operator ~base:Modules.debug "is_ready" ~category:`Input
+    ~descr:
+      "A source that always produces an empty frame when the underlying source \
+       is not ready, used for testing and debugging."
+    ~flags:[`Experimental] ~return_t
+    [("", Lang.source_t return_t, None, None)]
+    (fun p ->
+      let s = Lang.to_source (List.assoc "" p) in
+      new is_ready s)
