@@ -259,7 +259,7 @@ class dynamic ?(name = "request.dynamic") ~retry_delay ~available ~prefetch
 
     (** This method should be called whenever the feeding task has a new
         opportunity to feed the queue, in case it is sleeping. *)
-    method private notify_new_request =
+    method notify_new_request =
       match Atomic.get state with
         | `Started (d, { notify }) when d <= Unix.gettimeofday () -> notify ()
         | _ -> ()
@@ -344,7 +344,6 @@ class dynamic ?(name = "request.dynamic") ~retry_delay ~available ~prefetch
 
 let _ =
   let return_t = Lang.frame_t (Lang.univ_t ()) Frame.Fields.empty in
-  let log = Log.make ["request"; "dynamic"] in
   Lang.add_operator ~base:Modules.request "dynamic" ~category:`Input
     ~descr:"Play request dynamically created by a given function."
     [
@@ -380,66 +379,86 @@ let _ =
     ]
     ~meth:
       [
-        ( "fetch",
-          ([], Lang.fun_t [] Lang.bool_t),
-          "Try feeding the queue with a new request. Returns `true` if \
-           successful. This method can take long to return and should usually \
-           be run in a separate thread.",
-          fun s ->
-            Lang.val_fun [] (fun _ ->
-                match s#fetch with
-                  | `Finished -> Lang.bool true
-                  | `Retry ->
-                      log#important "Fetch failed";
-                      Lang.bool false) );
-        ( "queue",
-          ([], Lang.fun_t [] (Lang.list_t Request.Value.t)),
-          "Get the requests currently in the queue.",
-          fun s ->
-            Lang.val_fun [] (fun _ ->
-                let requests =
-                  List.map
-                    (fun r -> Request.Value.to_value r.request)
-                    (Queue.elements s#queue)
-                in
-                Lang.list requests) );
-        ( "add",
-          ([], Lang.fun_t [(false, "", Request.Value.t)] Lang.bool_t),
-          "Add a request to the queue. Requests are resolved before being \
-           added. Returns `true` if the request was successfully added.",
-          fun s ->
-            Lang.val_fun
-              [("", "", None)]
-              (fun p ->
-                Lang.bool
-                  (s#add
-                     {
-                       request = Request.Value.of_value (List.assoc "" p);
-                       expired = false;
-                     })) );
-        ( "set_queue",
-          ([], Lang.fun_t [(false, "", Lang.list_t Request.Value.t)] Lang.unit_t),
-          "Set the queue of requests. Requests are resolved before being added \
-           to the queue. You are responsible for destroying the requests \
-           currently in the queue.",
-          fun s ->
-            Lang.val_fun
-              [("", "", None)]
-              (fun p ->
-                let l =
-                  List.map Request.Value.of_value
-                    (Lang.to_list (List.assoc "" p))
-                in
-                s#set_queue l;
-                Lang.unit) );
-        ( "current",
-          ([], Lang.fun_t [] (Lang.nullable_t Request.Value.t)),
-          "Get the request currently being played.",
-          fun s ->
-            Lang.val_fun [] (fun _ ->
-                match s#current with
-                  | None -> Lang.null
-                  | Some c -> Request.Value.to_value c.req) );
+        {
+          name = "fetch";
+          scheme = ([], Lang.fun_t [] Lang.unit_t);
+          descr =
+            "Notify the source to queue a new request. If `synchronous` is \
+             `false` (the default), this method returns immediately and a new \
+             request is fetched in the background. If `synchronous` is `true`, \
+             can take long to return and should usually be run in a separate \
+             thread.";
+          value =
+            (fun s ->
+              Lang.val_fun [] (fun _ ->
+                  s#notify_new_request;
+                  Lang.unit));
+        };
+        {
+          name = "queue";
+          scheme = ([], Lang.fun_t [] (Lang.list_t Request.Value.t));
+          descr = "Get the requests currently in the queue.";
+          value =
+            (fun s ->
+              Lang.val_fun [] (fun _ ->
+                  let requests =
+                    List.map
+                      (fun r -> Request.Value.to_value r.request)
+                      (Queue.elements s#queue)
+                  in
+                  Lang.list requests));
+        };
+        {
+          name = "add";
+          scheme = ([], Lang.fun_t [(false, "", Request.Value.t)] Lang.bool_t);
+          descr =
+            "Add a request to the queue. Requests are resolved before being \
+             added. Returns `true` if the request was successfully added.";
+          value =
+            (fun s ->
+              Lang.val_fun
+                [("", "", None)]
+                (fun p ->
+                  Lang.bool
+                    (s#add
+                       {
+                         request = Request.Value.of_value (List.assoc "" p);
+                         expired = false;
+                       })));
+        };
+        {
+          name = "set_queue";
+          scheme =
+            ( [],
+              Lang.fun_t [(false, "", Lang.list_t Request.Value.t)] Lang.unit_t
+            );
+          descr =
+            "Set the queue of requests. Requests are resolved before being \
+             added to the queue. You are responsible for destroying the \
+             requests currently in the queue.";
+          value =
+            (fun s ->
+              Lang.val_fun
+                [("", "", None)]
+                (fun p ->
+                  let l =
+                    List.map Request.Value.of_value
+                      (Lang.to_list (List.assoc "" p))
+                  in
+                  s#set_queue l;
+                  Lang.unit));
+        };
+        {
+          name = "current";
+          scheme = ([], Lang.fun_t [] (Lang.nullable_t Request.Value.t));
+          descr = "Get the request currently being played.";
+          value =
+            (fun s ->
+              Lang.val_fun [] (fun _ ->
+                  match s#current with
+                    | None -> Lang.null
+                    | Some c -> Request.Value.to_value c.req));
+        };
       ]
     ~return_t
     (fun p ->
