@@ -1,6 +1,6 @@
-(** Specialization of Frame for content kinds restricted to audio,
-  * with a fixed number of channels.
-  * All units are in audio samples instead of main clock ticks. *)
+(** Specialization of Frame for content kinds restricted to audio, with a fixed
+    number of channels. All units are in audio samples instead of main clock
+    ticks. *)
 
 (** {2 Types} *)
 
@@ -12,8 +12,7 @@ type t = Frame.t
 val duration : unit -> float
 
 (** {2 Track marks}
-  * Track marks are track limits.
-  * Everything below is in samples. *)
+    Track marks are track limits. Everything below is in samples. *)
 
 (** Size of an audio frame. *)
 val size : unit -> int
@@ -23,12 +22,11 @@ val position : t -> int
 
 (** {2 Helpers} *)
 
-(** Get PCM content. Raises [Content.Invalid]
-  * if frame content is not pcm. *)
+(** Get PCM content. Raises [Content.Invalid] if frame content is not pcm. *)
 val pcm : t -> Content.Audio.data
 
-(** Get content in S16LE format. Raises [Content.Invalid]
-  * if frame content is not pcm. *)
+(** Get content in S16LE format. Raises [Content.Invalid] if frame content is
+    not pcm. *)
 val s16le : t -> string
 
 (** {2 Sound processing} *)
