@@ -24,17 +24,6 @@ open Mm
 
 (** Generic content registration API. *)
 
-type 'a chunk = 'a Content_base.chunk = {
-  data : 'a;
-  offset : int;
-  length : int option;
-}
-
-type ('a, 'b) chunks = ('a, 'b) Content_base.chunks = {
-  mutable params : 'a;
-  mutable chunks : 'b chunk list;
-}
-
 module Contents = Content_base.Contents
 
 (* Raised during any invalid operation below. *)
@@ -62,6 +51,8 @@ module type ContentSpecs = sig
 
   val params : data -> params
   val merge : params -> params -> params
+
+  (* [compatible src dst] *)
   val compatible : params -> params -> bool
   val string_of_params : params -> string
 
@@ -84,7 +75,6 @@ module type Content = sig
   val is_data : Contents.data -> bool
   val lift_data : ?offset:int -> ?length:int -> data -> Contents.data
   val get_data : Contents.data -> data
-  val get_chunked_data : Contents.data -> (params, data) chunks
 
   (** Format *)
 
@@ -112,7 +102,6 @@ type data = Contents.data
 (** Data *)
 
 val make : ?length:int -> format -> data
-val fill : data -> int -> data -> int -> int -> unit
 val sub : data -> int -> int -> data
 val truncate : data -> int -> data
 val copy : data -> data
@@ -125,6 +114,8 @@ val is_empty : data -> bool
 val format : data -> format
 val duplicate : format -> format
 val merge : format -> format -> unit
+
+(* [compatible src dst] *)
 val compatible : format -> format -> bool
 val string_of_format : format -> string
 
