@@ -1,7 +1,7 @@
 (*****************************************************************************
 
-  Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2023 Savonet team
+  Liquidsoap, a programmable stream generator.
+  Copyright 2003-2024 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -35,13 +35,13 @@ let dssi_enable =
 let dssi_load =
   try
     let venv = Unix.getenv "LIQ_DSSI_LOAD" in
-    Pcre.split ~pat:":" venv
+    String.split_on_char ':' venv
   with Not_found -> []
 
 let plugin_dirs =
   try
     let path = Unix.getenv "LIQ_DSSI_PATH" in
-    Pcre.split ~pat:":" path
+    String.split_on_char ':' path
   with Not_found -> ["/usr/lib/dssi"; "/usr/local/lib/dssi"]
 
 (* Number of channels to synthesize when in all mode *)
@@ -52,9 +52,9 @@ class dssi ?chan plugin descr outputs params source =
   object
     inherit operator ~name:"dssi" [source]
     method seek_source = source#seek_source
-    method stype = source#stype
+    method fallible = source#fallible
     method remaining = source#remaining
-    method private _is_ready = source#is_ready
+    method private can_generate_frame = source#is_ready
     method self_sync = source#self_sync
     method abort_track = source#abort_track
 
@@ -71,19 +71,16 @@ class dssi ?chan plugin descr outputs params source =
     initializer
       Array.iter (fun inst -> Ladspa.Descriptor.activate inst) (snd di)
 
-    method private get_frame buf =
+    method private generate_frame =
+      let b =
+        Content.Audio.get_data (source#get_mutable_content Frame.Fields.audio)
+      in
+      let alen = source#frame_audio_position in
       let descr, inst = di in
-      let offset = Frame.position buf in
-      let position =
-        source#get buf;
-        Frame.position buf
+      let evs =
+        Content.Midi.get_data (Frame.get source#get_frame Frame.Fields.midi)
       in
-      let b = AFrame.pcm buf in
-      let evs = MFrame.midi buf in
       (* Now convert everything to audio samples. *)
-      let offset = Frame.audio_of_main offset in
-      let position = Frame.audio_of_main position in
-      let len = position - offset in
       let evs =
         let dssi_of_midi (t, e) =
           let t = Frame.audio_of_main (Frame.main_of_midi t) in
@@ -104,7 +101,7 @@ class dssi ?chan plugin descr outputs params source =
                     (List.filter_map dssi_of_midi (MIDI.data evs.(chan))))
       in
       assert (Array.length outputs = Array.length b);
-      let ba = Audio.to_ba b offset len in
+      let ba = Audio.to_ba b 0 alen in
       Array.iter
         (fun inst ->
           List.iter
@@ -114,12 +111,13 @@ class dssi ?chan plugin descr outputs params source =
             Ladspa.Descriptor.connect_port inst outputs.(c) ba.(c)
           done)
         inst;
-      try Descriptor.run_multiple_synths descr ~adding:true inst len evs
-      with Descriptor.Not_implemented ->
-        for i = 0 to (if chan = None then all_chans else 1) - 1 do
-          Descriptor.run_synth ~adding:true descr inst.(i) len evs.(i)
-        done;
-        Audio.copy_from_ba ba b offset len
+      (try Descriptor.run_multiple_synths descr ~adding:true inst alen evs
+       with Descriptor.Not_implemented ->
+         for i = 0 to (if chan = None then all_chans else 1) - 1 do
+           Descriptor.run_synth ~adding:true descr inst.(i) alen evs.(i)
+         done;
+         Audio.copy_from_ba ba b 0 alen);
+      source#set_frame_data Frame.Fields.audio Content.Audio.lift_data b
   end
 
 let dssi = Lang.add_module "dssi"
@@ -128,7 +126,9 @@ let synth_all_dssi = Lang.add_module ~base:Modules.synth_all "dssi"
 
 let register_descr plugin_name descr_n descr outputs =
   let ladspa_descr = Descriptor.ladspa descr in
-  let liq_params, params = Ladspa_op.params_of_descr ladspa_descr in
+  let liq_params, params =
+    Ladspa_op.params_of_controls (Ladspa_op.get_control_ports ladspa_descr)
+  in
   let chans = Array.length outputs in
   let frame_t =
     Lang.frame_t (Lang.univ_t ())
@@ -194,9 +194,8 @@ let register_plugin ?(log_errors = false) pname =
           log#important
             "Plugin %s has inputs, don't know how to handle them for now."
             (Ladspa.Descriptor.label ladspa_descr))
-      descr
-    (* TODO: Unloading plugins makes liq segv. Don't do it for now. *)
-    (* Plugin.unload p *)
+      descr;
+    Plugin.unload p
   with Plugin.Not_a_plugin ->
     if log_errors then log#important "File \"%s\" is not a plugin!" pname
 
