@@ -14,6 +14,7 @@ type socket =
   ; wait_for : ?log:(string -> unit) -> event -> float -> unit
   ; write : Bytes.t -> int -> int -> int
   ; read : Bytes.t -> int -> int -> int
+  ; closed : bool
   ; close : unit >
 
 and server =
@@ -33,6 +34,7 @@ and transport =
       socket
   ; server : server >
 
+let socket_log = Log.make ["socket"]
 let connect = Cry.unix_connect
 
 let rec accept ?timeout sock =
@@ -57,17 +59,41 @@ let rec accept ?timeout sock =
     | Unix.Unix_error (Unix.EWOULDBLOCK, _, _) when has_timeout ->
         check_timeout ()
 
-let rec unix_socket fd =
+let rec unix_socket ~pos fd =
   let s = Cry.unix_socket fd in
-  object
-    method typ = s#typ
-    method file_descr = fd
-    method transport = unix_transport ()
-    method wait_for = s#wait_for
-    method write = s#write
-    method read = s#read
-    method close = s#close
-  end
+  let closed = Atomic.make false in
+  let finalise () =
+    if not (Atomic.get closed) then (
+      let pos =
+        match pos with
+          | [] -> "unknown"
+          | _ ->
+              String.concat ", " (List.map (fun pos -> Pos.to_string pos) pos)
+      in
+      socket_log#critical
+        "File descriptor closed during garbage collection, you must have a \
+         file descriptor leak in your application! File descriptor opened at \
+         position: %s"
+        pos;
+      try s#close with _ -> ())
+  in
+  let s =
+    object
+      method typ = s#typ
+      method file_descr = fd
+      method transport = unix_transport ()
+      method wait_for = s#wait_for
+      method write = s#write
+      method read = s#read
+      method closed = Atomic.get closed
+
+      method close =
+        Atomic.set closed true;
+        s#close
+    end
+  in
+  Gc.finalise_last finalise s;
+  s
 
 and unix_transport () =
   object (self)
@@ -77,7 +103,7 @@ and unix_transport () =
 
     method connect ?bind_address ?timeout ?prefer host port =
       let fd = Cry.unix_connect ?bind_address ?timeout ?prefer host port in
-      unix_socket fd
+      unix_socket ~pos:[] fd
 
     method server =
       object
@@ -85,7 +111,7 @@ and unix_transport () =
 
         method accept ?timeout fd =
           let fd, addr = accept ?timeout fd in
-          (unix_socket fd, addr)
+          (unix_socket ~pos:[] fd, addr)
       end
   end
 
