@@ -22,7 +22,7 @@
 
 let null =
   let a = Lang.univ_t () in
-  Lang.add_builtin "null" ~category:`Programming
+  Lang.add_builtin "_null" ~category:`Programming
     ~descr:"Create a nullable value."
     [("", Lang.nullable_t a, Some Lang.null, Some "Value to make nullable.")]
     (Lang.nullable_t a)
