@@ -24,3 +24,13 @@
 exception Invalid_value of Value.t * string
 
 exception Clock_conflict of (Pos.Option.t * string * string)
+
+type clock_main = {
+  pos : Pos.Option.t;
+  left_main : string;
+  left_child : string;
+  right_main : string;
+  right_child : string;
+}
+
+exception Clock_main of clock_main
