@@ -52,20 +52,23 @@ let rec prepend_parsed_stdlib =
     the full env using it. Next, we append the user script to the resulting term
     and typecheck the user script only using the standard library environment.
 *)
-let prepare ?libs ~cache ~error_on_no_stdlib ~deprecated parsed_term =
-  let libs =
-    match libs with
-      | Some libs -> libs
-      | None -> Runtime.libs ~error_on_no_stdlib ~deprecated ()
+let prepare ~stdlib ~cache ~error_on_no_stdlib ~deprecated parsed_term =
+  let stdlib =
+    match stdlib with
+      | Some stdlib -> stdlib
+      | None ->
+          let dir = !Hooks.liq_libs_dir () in
+          Filename.concat dir "stdlib.liq"
   in
+  let libs = Runtime.libs ~stdlib ~error_on_no_stdlib ~deprecated () in
   let script = List.fold_left (Printf.sprintf "%s\n%%include %S") "" libs in
   let lexbuf = Sedlexing.Utf8.from_string script in
   let parsed_stdlib, stdlib =
     Runtime.report ~lexbuf:(Some lexbuf)
       ~default:(fun () -> raise Runtime.Error)
-      (fun ~throw:_ () ->
+      (fun ~throw () ->
         let parsed_stdlib = Term_reducer.mk_expr Term_reducer.program lexbuf in
-        (parsed_stdlib, Term_reducer.to_term parsed_stdlib))
+        (parsed_stdlib, Term_reducer.to_term ~throw parsed_stdlib))
   in
   let append () =
     let stdlib = append_ref stdlib in
@@ -79,7 +82,7 @@ let prepare ?libs ~cache ~error_on_no_stdlib ~deprecated parsed_term =
     let checked_term =
       Runtime.report ~lexbuf:None
         ~default:(fun () -> raise Runtime.Error)
-        (fun ~throw:_ () -> Term_reducer.to_term parsed_term)
+        (fun ~throw () -> Term_reducer.to_term ~throw parsed_term)
     in
     let full_term = prepend_stdlib ~term:checked_term stdlib in
     { Runtime.checked_term; full_term; env }
