@@ -84,11 +84,16 @@ type var = Type_base.var = {
 
 type scheme = var list * t
 
+type meth_doc = Type_base.meth_doc = {
+  meth_descr : string;
+  category : [ `Method | `Callback ];
+}
+
 type meth = Type_base.meth = {
   meth : string;
   optional : bool;
   scheme : scheme;
-  doc : string;
+  doc : meth_doc;
   json_name : string option;
 }
 
@@ -148,6 +153,7 @@ val invokes : t -> string list -> var list * t
 val meth :
   ?pos:Pos.t ->
   ?json_name:string ->
+  ?category:[ `Method | `Callback ] ->
   ?optional:bool ->
   string ->
   scheme ->
@@ -158,7 +164,15 @@ val meth :
 (** Type of references on a given type. *)
 val reference : ?pos:Pos.t -> t -> t
 
-val meths : ?pos:Pos.t -> string list -> scheme -> t -> t
+(** [invoke] is used to raise proper exception in [Typechecking]. *)
+val meths :
+  ?invoke:(t -> string -> scheme) ->
+  ?pos:Pos.t ->
+  string list ->
+  scheme ->
+  t ->
+  t
+
 val split_meths : t -> meth list * t
 val hide_meth : string -> t -> t
 val opt_meth : string -> t -> t
