@@ -46,6 +46,7 @@ let () =
                  as_variable = None;
                  typ = Type.var ();
                  default = None;
+                 pos = None;
                };
              ];
            body =
@@ -53,7 +54,9 @@ let () =
                (`App (_if, [("", cond); ("then", _then); ("else", _else)]));
          })
   in
-  Typechecking.check ~throw:(fun exn -> raise exn) f;
+  Typechecking.check
+    ~throw:(fun ~bt exn -> Printexc.raise_with_backtrace exn bt)
+    ~check_top_level_override:false f;
   match (Type.deref f.Term.t).Type.descr with
     | Type.Arrow ([(false, "x", _)], t) -> (
         let meths, _ = Type.split_meths t in
@@ -85,7 +88,9 @@ let () =
     }
   in
   try
-    Typechecking.check ~throw:(fun exn -> raise exn) invoke;
+    Typechecking.check
+      ~throw:(fun ~bt exn -> Printexc.raise_with_backtrace exn bt)
+      ~check_top_level_override:false invoke;
     raise Failed
   with
     | Failed -> raise Failed
