@@ -1,6 +1,6 @@
 class start_stop_test _state start stop =
   object
-    inherit Start_stop.base ~on_start:(fun () -> ()) ~on_stop:(fun () -> ())
+    inherit Start_stop.base
     method! state = _state
     method start = start ()
     method stop = stop ()
